@@ -8,24 +8,61 @@
 #         Jerome Carnis, carnis_jerome@yahoo.fr
 
 """
-Detector classes.
+Implementation of the detector classes.
 
-These classes handles the detector config used for data acquisition. The available
-detectors are:
+These classes handle the detector-dependent paths and data filenames, on top of the
+detector-dependent properties (e.g., number and size of the pixels, gaps between
+tiles...). Generic methods are implemented in the abstract base class Detector, and
+detector-dependent properties need to be implemented in each child class (they are
+decoracted by @abstractmethod in the base class, they are indicated using @ in the
+following diagram).
 
-- Maxipix
-- Eiger2M
-- Eiger4M
-- Timepix
-- Merlin
-- Dummy
+.. mermaid::
+  :align: center
+
+  classDiagram
+    class Detector{
+      +str name
+      +tuple binning
+      +str datadir
+      +int nb_pixel_x
+      +int nb_pixel_y
+      +float pixelsize_x
+      +float pixelsize_y
+      +dict params
+      +tuple preprocessing_binning
+      +tuple roi
+      +str rootdir
+      +str sample_name
+      +str savedir
+      +str scandir
+      +tuple sum_roi
+      +str template_file
+      +str template_imagefile
+      unbinned_pixel_number(@)
+      unbinned_pixel_size(@)
+      _background_subtraction()
+      _flatfield_correction()
+      _hotpixels_correction()
+      _linearity_correction()
+      _mask_gaps()
+      _saturation_correction()
+      counter()
+      linearity_func()
+      mask_detector()
+  }
+    ABC <|-- Detector
+
+API Reference
+-------------
 
 """
 
 from abc import ABC, abstractmethod
 import numpy as np
-from numbers import Real
+from numbers import Real, Integral
 import os
+import pathlib
 
 from bcdi.utils import validation as valid
 
@@ -110,12 +147,7 @@ class Detector(ABC):
         # load the kwargs
         self.preprocessing_binning = kwargs.get("preprocessing_binning") or (1, 1, 1)
         self.offsets = kwargs.get("offsets")  # delegate the test to xrayutilities
-        linearity_func = kwargs.get("linearity_func")
-        if linearity_func is not None and not callable(linearity_func):
-            raise TypeError(
-                f"linearity_func should be a function, got {type(linearity_func)}"
-            )
-        self._linearity_func = linearity_func
+        self.linearity_func = kwargs.get("linearity_func")
 
         # load other positional arguments
         self.binning = binning
@@ -185,8 +217,21 @@ class Detector(ABC):
             allow_none=True,
             name="Detector.datadir",
         )
+        if value is not None and not os.path.isdir(value):
+            raise ValueError(f"The directory {value} does not exist")
         self._datadir = value
 
+    @property
+    def linearity_func(self):
+        """Correction of the non-linearity of the detector with high incoming flux."""
+        return self._linearity_func
+
+    @linearity_func.setter
+    def linearity_func(self, value):
+        if value is not None and not callable(value):
+            raise TypeError(f"linearity_func should be a function, got {type(value)}")
+        self._linearity_func = value
+
     @property
     def name(self):
         """Name of the detector."""
@@ -291,11 +336,13 @@ class Detector(ABC):
             value = [0, self.nb_pixel_y, 0, self.nb_pixel_x]
         valid.valid_container(
             value,
-            container_types=(tuple, list),
+            container_types=(tuple, list, np.ndarray),
             length=4,
             item_types=int,
             name="Detector.roi",
         )
+        if value[1] <= value[0] or value[3] <= value[2]:
+            raise ValueError("roi coordinates should be increasing in x and y")
         self._roi = value
 
     @property
@@ -312,6 +359,8 @@ class Detector(ABC):
             allow_none=True,
             name="Detector.rootdir",
         )
+        if value is not None and not os.path.isdir(value):
+            raise ValueError(f"The directory {value} does not exist")
         self._rootdir = value
 
     @property
@@ -344,6 +393,8 @@ class Detector(ABC):
             allow_none=True,
             name="Detector.savedir",
         )
+        if value is not None:
+            pathlib.Path(value).mkdir(parents=True, exist_ok=True)
         self._savedir = value
 
     @property
@@ -352,6 +403,8 @@ class Detector(ABC):
         if self.datadir:
             dir_path = os.path.abspath(os.path.join(self.datadir, os.pardir)) + "/"
             return dir_path.replace("\\", "/")
+        else:
+            return None
 
     @property
     def sum_roi(self):
@@ -365,10 +418,7 @@ class Detector(ABC):
     @sum_roi.setter
     def sum_roi(self, value):
         if not value:  # None or empty list/tuple
-            if not self.roi:
-                value = [0, self.nb_pixel_y, 0, self.nb_pixel_x]
-            else:
-                value = self.roi
+            value = self.roi
         valid.valid_container(
             value,
             container_types=(tuple, list),
@@ -376,6 +426,8 @@ class Detector(ABC):
             item_types=int,
             name="Detector.sum_roi",
         )
+        if value[1] <= value[0] or value[3] <= value[2]:
+            raise ValueError("roi coordinates should be increasing in x and y")
         self._sum_roi = value
 
     @property
@@ -418,11 +470,13 @@ class Detector(ABC):
 
         Convention: (vertical, horizontal)
         """
+        return 1, 1
 
     @property
     @abstractmethod
     def unbinned_pixel_size(self):
         """Pixel size (vertical, horizontal) of the unbinned detector in meters."""
+        return 1, 1
 
     def __repr__(self):
         """Representation string of the Detector instance."""
@@ -454,8 +508,6 @@ class Detector(ABC):
         :param background: None or a 2D numpy array
         :return: the corrected data array
         """
-        if data.ndim != 2:
-            raise ValueError("data should be a 2D array")
         if background is not None:
             valid.valid_ndarray((data, background), ndim=2)
             return data - background
@@ -464,7 +516,7 @@ class Detector(ABC):
     @staticmethod
     def _flatfield_correction(data, flatfield):
         """
-        Apply flatfield correction to the data.
+        Apply flatfield correction to the data (multiplication).
 
         :param data: a 2D numpy ndarray
         :param flatfield: None or a 2D numpy array
@@ -502,13 +554,11 @@ class Detector(ABC):
         :param data: a 2D numpy array
         :return: the corrected data array
         """
-        if self._linearity_func is not None:
-            if not callable(self._linearity_func):
-                raise TypeError("linearity_function is not callable")
+        if self.linearity_func is not None:
             valid.valid_ndarray(data, ndim=2)
             data = data.astype(float)
             nby, nbx = data.shape
-            return self._linearity_func(data.flatten()).reshape((nby, nbx))
+            return self.linearity_func(data.flatten()).reshape((nby, nbx))
         return data
 
     def mask_detector(
@@ -532,9 +582,8 @@ class Detector(ABC):
         """
         valid.valid_ndarray((data, mask), ndim=2)
 
-        # linearity correctiondata
-        if self._linearity_func is not None:
-            data = self._linearity_func(data)
+        # linearity correction, returns the data itself if linearity_func is None
+        data = self._linearity_correction(data)
 
         # flatfield correction
         data = self._flatfield_correction(data=data, flatfield=flatfield)
@@ -554,8 +603,7 @@ class Detector(ABC):
 
         return data, mask
 
-    @staticmethod
-    def _mask_gaps(data, mask):
+    def _mask_gaps(self, data, mask):
         """
         Mask the gaps between sensors in the detector.
 
@@ -567,14 +615,18 @@ class Detector(ABC):
          - the updated mask
 
         """
-        valid.valid_ndarray((data, mask), ndim=2)
+        valid.valid_ndarray(
+            (data, mask), ndim=2, shape=self.unbinned_pixel_number, fix_shape=True
+        )
         return data, mask
 
     def _saturation_correction(self, data, mask, nb_frames):
         """
         Mask pixels above a certain threshold.
 
-        This is detector dependent.
+        This is detector dependent. If a 2D frames was obtained by summing a series of
+        frames (e.g. series measurement at P10), the threshold is multiplied
+        accordingly.
 
         :param data: a 2D numpy array
         :param mask: a 2D numpy array of the same shape as data
@@ -605,8 +657,7 @@ class Maxipix(Detector):
         # is used at several beamlines
         self.saturation_threshold = 1e6
 
-    @staticmethod
-    def _mask_gaps(data, mask):
+    def _mask_gaps(self, data, mask):
         """
         Mask the gaps between sensors in the detector.
 
@@ -618,7 +669,9 @@ class Maxipix(Detector):
          - the updated mask
 
         """
-        valid.valid_ndarray((data, mask), ndim=2)
+        valid.valid_ndarray(
+            (data, mask), ndim=2, shape=self.unbinned_pixel_number, fix_shape=True
+        )
 
         data[:, 255:261] = 0
         data[255:261, :] = 0
@@ -651,8 +704,7 @@ class Eiger2M(Detector):
         # is used at several beamlines
         self.saturation_threshold = 1e6
 
-    @staticmethod
-    def _mask_gaps(data, mask):
+    def _mask_gaps(self, data, mask):
         """
         Mask the gaps between sensors in the detector.
 
@@ -664,7 +716,9 @@ class Eiger2M(Detector):
          - the updated mask
 
         """
-        valid.valid_ndarray((data, mask), ndim=2)
+        valid.valid_ndarray(
+            (data, mask), ndim=2, shape=self.unbinned_pixel_number, fix_shape=True
+        )
 
         data[:, 255:259] = 0
         data[:, 513:517] = 0
@@ -719,8 +773,7 @@ class Eiger4M(Detector):
         super().__init__(name=name, **kwargs)
         self.saturation_threshold = 4000000000
 
-    @staticmethod
-    def _mask_gaps(data, mask):
+    def _mask_gaps(self, data, mask):
         """
         Mask the gaps between sensors in the detector.
 
@@ -732,7 +785,9 @@ class Eiger4M(Detector):
          - the updated mask
 
         """
-        valid.valid_ndarray((data, mask), ndim=2)
+        valid.valid_ndarray(
+            (data, mask), ndim=2, shape=self.unbinned_pixel_number, fix_shape=True
+        )
 
         data[:, 0:1] = 0
         data[:, -1:] = 0
@@ -796,8 +851,7 @@ class Merlin(Detector):
         super().__init__(name=name, **kwargs)
         self.saturation_threshold = 1e6
 
-    @staticmethod
-    def _mask_gaps(data, mask):
+    def _mask_gaps(self, data, mask):
         """
         Mask the gaps between sensors in the detector.
 
@@ -809,7 +863,9 @@ class Merlin(Detector):
          - the updated mask
 
         """
-        valid.valid_ndarray((data, mask), ndim=2)
+        valid.valid_ndarray(
+            (data, mask), ndim=2, shape=self.unbinned_pixel_number, fix_shape=True
+        )
 
         data[:, 255:260] = 0
         data[255:260, :] = 0
@@ -855,11 +911,13 @@ class Dummy(Detector):
             name="custom_pixelsize",
         )
         self.custom_pixelnumber = kwargs.get("custom_pixelnumber")
+        if isinstance(self.custom_pixelnumber, np.ndarray):
+            self.custom_pixelnumber = list(self.custom_pixelnumber)
         valid.valid_container(
             self.custom_pixelnumber,
-            container_types=(list, tuple, np.ndarray),
+            container_types=(list, tuple),
             length=2,
-            item_types=int,
+            item_types=Integral,
             min_excluded=0,
             allow_none=True,
             name="custom_pixelnumber",
@@ -877,7 +935,7 @@ class Dummy(Detector):
             val is not None for val in self.custom_pixelnumber
         ):
             return self.custom_pixelnumber
-        print(f"Defaulting the pixel size to {516, 516}")
+        print(f"Defaulting the pixel number to {516, 516}")
         return 516, 516
 
     @property
