@@ -7,14 +7,21 @@
 #       authors:
 #         Jerome Carnis, carnis_jerome@yahoo.fr
 
-"""Setup class that defines the experimental geometry."""
-from collections.abc import Sequence
-import gc
-from numbers import Real
-import numpy as np
-import pathlib
-from scipy.interpolate import RegularGridInterpolator
+"""
+Setup class that defines the experimental geometry.
 
+You can think of it as the public interface for the Beamline and Diffractometer child
+classes. A script would call a method from Setup, which would then retrieve the required
+beamline-dependent information from the child classes.
+"""
+from collections.abc import Sequence
+import datetime
+import multiprocessing as mp
+from numbers import Real, Integral
+import numpy as np
+from scipy.interpolate import griddata, RegularGridInterpolator
+import sys
+import time
 from ..graph import graph_utils as gu
 from ..utils import utilities as util
 from ..utils import validation as valid
@@ -231,11 +238,13 @@ class Setup:
         if not self._custom_scan:
             self._custom_images = None
         else:
+            if isinstance(value, np.ndarray):
+                value = list(value)
             valid.valid_container(
                 value,
                 container_types=(tuple, list),
                 min_length=1,
-                item_types=int,
+                item_types=Integral,
                 allow_none=True,
                 name="Setup.custom_images",
             )
@@ -636,6 +645,8 @@ class Setup:
         """Wavelength in meters."""
         if self.energy:
             return 12.398 * 1e-7 / self.energy  # in m
+        else:
+            return None
 
     def __repr__(self):
         """Representation string of the Setup instance."""
@@ -652,8 +663,8 @@ class Setup:
             f"pixel_size={self.detector.unbinned_pixel_size}, "
             f"direct_beam={self.direct_beam}, "
             f"sample_offsets={self.diffractometer.sample_offsets}, "
-            f"filtered_data={self.filtered_data}, "
-            f"custom_scan={self.custom_scan},\n"
+            f"filtered_data={self.filtered_data},\n"
+            f"custom_scan={self.custom_scan}, "
             f"custom_images={self.custom_images},\n"
             f"custom_monitor={self.custom_monitor},\n"
             f"custom_motors={self.custom_motors},\n"
@@ -663,6 +674,65 @@ class Setup:
             f"is_series={self.is_series})"
         )
 
+    def calc_qvalues_xrutils(self, logfile, hxrd, nb_frames, **kwargs):
+        """
+        Calculate the 3D q values of the BCDI scan using xrayutilities.
+
+        :param logfile: the logfile created in Setup.create_logfile()
+        :param hxrd: an initialized xrayutilities HXRD object used for the
+         orthogonalization of the dataset
+        :param nb_frames: length of axis 0 in the 3D dataset. If the data was cropped
+         or padded, it may be different from the original length len(frames_logical)
+        :param kwargs:
+
+         - 'scan_number': the scan number to load
+         - 'frames_logical': array of length the number of measured frames.
+           In case of cropping/padding the number of frames changes. A frame whose
+           index is set to 1 means that it is used, 0 means not used, -1 means padded
+           (added) frame
+
+        :return:
+         - qx, qz, qy components for the dataset. xrayutilities uses the xyz crystal
+           frame: for incident angle = 0, x is downstream, y outboard, and z vertical
+           up. The output of hxrd.Ang2Q.area is qx, qy, qz is this order. If q values
+           seem wrong, check if diffractometer angles have default values set at 0,
+           otherwise use the parameter setup.diffractometer.sample_offsets to correct it
+         - updated frames_logical
+
+        """
+        # check some parameters
+        frames_logical = kwargs.get("frames_logical")
+        valid.valid_1d_array(
+            frames_logical,
+            allow_none=True,
+            allowed_types=Integral,
+            allowed_values=(-1, 0, 1),
+            name="frames_logical",
+        )
+        scan_number = kwargs.get("scan_number")
+        if not isinstance(scan_number, int):
+            raise TypeError(
+                "scan_number should be an integer, got " f"{type(scan_number)}"
+            )
+
+        # process motor positions
+        processed_positions = self._beamline.process_positions(
+            setup=self,
+            logfile=logfile,
+            nb_frames=nb_frames,
+            scan_number=scan_number,
+            frames_logical=frames_logical,
+        )
+
+        # calculate q values
+        qx, qy, qz = hxrd.Ang2Q.area(
+            *processed_positions[:-1],
+            en=processed_positions[-1],
+            delta=self.detector.offsets,
+        )
+        print("Use the parameter 'sample_offsets' to correct diffractometer values.\n")
+        return qx, qz, qy, frames_logical
+
     def create_logfile(self, scan_number, root_folder, filename):
         """
         Create the logfile, which can be a log/spec file or the data itself.
@@ -676,12 +746,16 @@ class Setup:
          'alias_dict.txt' for SIXS
         :return: logfile
         """
+        if self.custom_scan:
+            return None
+
         return self._beamline.create_logfile(
             scan_number=scan_number,
             root_folder=root_folder,
             filename=filename,
             datadir=self.detector.datadir,
             template_imagefile=self.detector.template_imagefile,
+            name=self.beamline,
         )
 
     def detector_frame(
@@ -741,11 +815,12 @@ class Setup:
                 title=title + " before interpolation\n",
             )
 
-        ortho_matrix = self.transformation_matrix(
+        ortho_matrix, _ = self.transformation_bcdi(
             array_shape=(nbz, nby, nbx),
             tilt_angle=self.tilt_angle,
             pixel_x=self.detector.unbinned_pixel_size[1],
             pixel_y=self.detector.unbinned_pixel_size[0],
+            direct_space=True,
         )
 
         ################################################
@@ -809,18 +884,152 @@ class Setup:
 
         return detector_obj
 
+    def grid_cylindrical(
+        self,
+        array,
+        rotation_angle,
+        direct_beam,
+        interp_angle,
+        interp_radius,
+        fill_value=np.nan,
+        comment="",
+        multiprocessing=False,
+    ):
+        """
+        Interpolate a tomographic dataset onto cartesian coordinates.
+
+        The initial 3D array is in cylindrical coordinates. There is no benefit from
+        multiprocessing, the data transfers are the limiting factor.
+
+        :param array: 3D array of intensities measured in the detector frame
+        :param rotation_angle: array, rotation angle values for the rocking scan
+        :param direct_beam: position in pixels of the rotation pivot in the direction
+         perpendicular to the rotation axis
+        :param interp_angle: 2D array, polar angles for the interpolation in a plane
+         perpendicular to the rotation axis
+        :param interp_radius: 2D array, polar radii for the interpolation in a plane
+         perpendicular to the rotation axis
+        :param fill_value: real number (np.nan allowed), fill_value parameter for the
+         RegularGridInterpolator
+        :param comment: a comment to be printed
+        :param multiprocessing: True to use multiprocessing
+        :return: the 3D array interpolated onto the 3D cartesian grid
+        """
+        valid.valid_ndarray(arrays=array, ndim=3)
+
+        def collect_result(result):
+            """
+            Process the result after asynchronous multiprocessing.
+
+            This callback function updates global arrays.
+
+            :param result: the output of interp_slice, containing the 2d interpolated
+             slice and the slice index
+            """
+            nonlocal interp_array, number_y, slices_done
+            slices_done = slices_done + 1
+            # result is a tuple: data, mask, counter, file_index
+            # stack the 2D interpolated frame along the rotation axis,
+            # taking into account the flip of the detector Y axis (pointing down)
+            # compare to the laboratory frame vertical axis (pointing up)
+            interp_array[:, number_y - (result[1] + 1), :] = result[0]
+            sys.stdout.write(
+                "\r    gridding progress: {:d}%".format(
+                    int(slices_done / number_y * 100)
+                )
+            )
+            sys.stdout.flush()
+
+        rotation_step = rotation_angle[1] - rotation_angle[0]
+        if rotation_step < 0:
+            # flip rotation_angle and the data accordingly, RegularGridInterpolator
+            # takes only increasing position vectors
+            rotation_angle = np.flip(rotation_angle)
+            array = np.flip(array, axis=0)
+
+        _, number_y, _ = array.shape
+        _, numx = interp_angle.shape  # data shape is (numx, numx) by construction
+        interp_array = np.zeros((numx, number_y, numx), dtype=array.dtype)
+        slices_done = 0
+
+        start = time.time()
+        if multiprocessing:
+            print(
+                "\nGridding",
+                comment,
+                ", number of processors used: ",
+                min(mp.cpu_count(), number_y),
+            )
+            mp.freeze_support()
+            pool = mp.Pool(
+                processes=min(mp.cpu_count(), number_y)
+            )  # use this number of processesu
+
+            for idx in range(number_y):
+                pool.apply_async(
+                    self.interp_2dslice,
+                    args=(
+                        array[:, idx, :],
+                        idx,
+                        rotation_angle,
+                        direct_beam,
+                        interp_angle,
+                        interp_radius,
+                        fill_value,
+                    ),
+                    callback=collect_result,
+                    error_callback=util.catch_error,
+                )
+                # interp_2dslice must be a pickable object,
+                # i.e. defined at the top level of the module
+
+            pool.close()
+            pool.join()
+            # postpones the execution of next line of code until all processes
+            # in the queue are done.
+
+        else:  # no multiprocessing
+            print("\nGridding", comment, ", no multiprocessing")
+            for idx in range(
+                number_y
+            ):  # loop over 2D frames perpendicular to the rotation axis
+                temp_array, _ = self.interp_2dslice(
+                    array=array[:, idx, :],
+                    slice_index=idx,
+                    rotation_angle=rotation_angle,
+                    direct_beam=direct_beam,
+                    interp_angle=interp_angle,
+                    interp_radius=interp_radius,
+                    fill_value=fill_value,
+                )
+
+                # stack the 2D interpolated frame along the rotation axis,
+                # taking into account the flip of the
+                # detector Y axis (pointing down) compare to the laboratory frame
+                # vertical axis (pointing up)
+                interp_array[:, number_y - (idx + 1), :] = temp_array
+                sys.stdout.write(
+                    "\rGridding progress: {:d}%".format(int((idx + 1) / number_y * 100))
+                )
+                sys.stdout.flush()
+
+        end = time.time()
+        print(
+            "\nTime ellapsed for gridding data:",
+            str(datetime.timedelta(seconds=int(end - start))),
+        )
+        return interp_array
+
     def init_paths(
         self,
         sample_name,
         scan_number,
         root_folder,
         save_dir,
-        specfile_name,
-        template_imagefile,
-        data_dirname=None,
+        specfile_name=None,
+        template_imagefile=None,
+        data_dir=None,
         save_dirname="result",
-        create_savedir=False,
-        verbose=False,
     ):
         """
         Init paths used for data processing and logging.
@@ -838,7 +1047,7 @@ class Setup:
          - ID01: name of the spec file without '.spec'
          - SIXS_2018 and SIXS_2019: None or full path of the alias dictionnary (e.g.
            root_folder+'alias_dict_2019.txt')
-         - empty string for all other beamlines
+         - None for all other beamlines
 
         :param template_imagefile: beamline-dependent template for the data files
 
@@ -850,62 +1059,64 @@ class Setup:
          - NANOMAX: '%06d.h5'
          - 34ID: 'Sample%dC_ES_data_51_256_256.npz'
 
-        :param data_dirname: name of the data folder, if None it will use the beamline
-         default, if it is an empty string, it will look for the data directly into
-         the scan folder (no subfolder)
+        :param data_dir: if None it will use the beamline default, otherwise it will
+         look for the data directly into that directory
         :param save_dirname: name of the saving folder, by default 'save_dir/result/'
          will be created
-        :param create_savedir: boolean, True to create the saving folder if it does
-         not exist
-        :param verbose: True to print the paths
         """
+        # check some parameters
         if not isinstance(scan_number, int):
             raise TypeError("scan_number should be an integer")
 
         if not isinstance(sample_name, str):
             raise TypeError("sample_name should be a string")
 
+        valid.valid_container(
+            specfile_name,
+            container_types=str,
+            min_length=1,
+            allow_none=True,
+            name="specfile_name",
+        )
+        valid.valid_container(
+            template_imagefile,
+            container_types=str,
+            min_length=1,
+            allow_none=True,
+            name="template_imagefile",
+        )
+
         # check that the provided folder names are not an empty string
         valid.valid_container(
-            save_dirname, container_types=str, min_length=1, name="Setup.init_paths"
+            save_dirname, container_types=str, min_length=1, name="save_dirname"
         )
         valid.valid_container(
-            data_dirname,
+            data_dir,
             container_types=str,
-            min_length=0,
+            min_length=1,
             allow_none=True,
-            name="Setup.init_paths",
+            name="data_dir",
         )
-        # update the detector instance
-        (
-            self.detector.rootdir,
-            self.detector.sample_name,
-            self.detector.template_file,
-        ) = (root_folder, sample_name, template_imagefile)
 
-        # create beamline-dependent path parameters
+        #############################################
+        # create beamline-dependent path parameters #
+        #############################################
         (
             homedir,
             default_dirname,
             specfile,
             template_imagefile,
         ) = self._beamline.init_paths(
+            root_folder=root_folder,
             sample_name=sample_name,
             scan_number=scan_number,
-            root_folder=root_folder,
-            save_dir=save_dir,
-            specfile_name=specfile_name,
             template_imagefile=template_imagefile,
-            data_dirname=data_dirname,
-            save_dirname=save_dirname,
+            specfile_name=specfile_name,
         )
 
         # define the data directory
-        if data_dirname is not None:
-            if len(data_dirname) == 0:  # no subfolder
-                datadir = homedir
-            else:
-                datadir = homedir + data_dirname
+        if data_dir is not None:
+            datadir = data_dir
         else:
             datadir = homedir + default_dirname
         if not datadir.endswith("/"):
@@ -918,29 +1129,16 @@ class Setup:
             savedir = homedir + save_dirname + "/"
         if not savedir.endswith("/"):
             savedir += "/"
-        if create_savedir:
-            pathlib.Path(savedir).mkdir(parents=True, exist_ok=True)
 
         # update the detector instance
         (
+            self.detector.rootdir,
             self.detector.savedir,
             self.detector.datadir,
+            self.detector.sample_name,
             self.detector.specfile,
             self.detector.template_imagefile,
-        ) = (savedir, datadir, specfile, template_imagefile)
-
-        if verbose:
-            if not self.custom_scan:
-                print(
-                    f"datadir = '{datadir}'\nsavedir = '{savedir}'\n"
-                    f"template_imagefile = '{template_imagefile}'\n"
-                )
-            else:
-                print(
-                    f"rootdir = '{root_folder}'\nsavedir = '{savedir}'\n"
-                    f"sample_name = '{self.detector.sample_name}'\n"
-                    f"template_imagefile = '{self.detector.template_file}'\n"
-                )
+        ) = (root_folder, savedir, datadir, sample_name, specfile, template_imagefile)
 
     def init_qconversion(self):
         """
@@ -964,6 +1162,143 @@ class Setup:
             diffractometer=self.diffractometer,
         )
 
+    @staticmethod
+    def interp_2dslice(
+        array,
+        slice_index,
+        rotation_angle,
+        direct_beam,
+        interp_angle,
+        interp_radius,
+        fill_value,
+    ):
+        """
+        Interpolate a 2D slice from a tomographic dataset onto cartesian coordinates.
+
+        The initial 3D array is in cylindrical coordinates.
+
+        :param array: 3D array of intensities measured in the detector frame
+        :param slice_index: the index along the rotation axis of the 2D slice in array
+         to interpolate
+        :param rotation_angle: array, rotation angle values for the rocking scan
+        :param direct_beam: position in pixels of the rotation pivot in the direction
+         perpendicular to the rotation axis
+        :param interp_angle: 2D array, polar angles for the interpolation in a plane
+         perpendicular to the rotation axis
+        :param interp_radius: 2D array, polar radii for the interpolation in a plane
+         perpendicular to the rotation axis
+        :param fill_value: real number (np.nan allowed), fill_value parameter for the
+         RegularGridInterpolator
+        :return: the interpolated slice, the slice index
+        """
+        valid.valid_ndarray(arrays=array, ndim=2)
+        # position of the experimental data points
+        number_x = array.shape[1]
+        rgi = RegularGridInterpolator(
+            (
+                rotation_angle * np.pi / 180,
+                np.arange(-direct_beam, -direct_beam + number_x, 1),
+            ),
+            array,
+            method="linear",
+            bounds_error=False,
+            fill_value=fill_value,
+        )
+
+        # interpolate the data onto the new points
+        tmp_array = rgi(
+            np.concatenate(
+                (
+                    interp_angle.reshape((1, interp_angle.size)),
+                    interp_radius.reshape((1, interp_angle.size)),
+                )
+            ).transpose()
+        )
+        tmp_array = tmp_array.reshape(interp_angle.shape)
+
+        return tmp_array, slice_index
+
+    def ortho_cdi(
+        self,
+        arrays,
+        cdi_angle,
+        fill_value=0,
+        correct_curvature=False,
+        debugging=False,
+    ):
+        """
+        Interpolate forward CDI data in the laboratory frame.
+
+        :param arrays: tuple of 3D arrays of the same shape (e.g.: reciprocal space
+         diffraction pattern and mask), in the detector frame
+        :param cdi_angle: 1D array of measurement angles in degrees
+        :param fill_value: tuple of real numbers (np.nan allowed), fill_value parameter
+         for the RegularGridInterpolator, same length as the number of arrays
+        :param correct_curvature: bool, True to take into account the curvature of the
+         Ewald sphere (uses griddata, very slow)
+        :param debugging: bool, True to see more plots
+        :return:
+         - an array (if a single array was provided) or a tuple of arrays interpolated
+           on an orthogonal grid (same length as the number of input arrays)
+         - a tuple of three 1D arrays for the q values (qx, qz, qy) where qx is
+           downstream, qz is vertical up and qy is outboard.
+         - a tuple of two integersfor the corrected position of the direct beam (V, H)
+
+        """
+        #########################
+        # check some parameters #
+        #########################
+        valid.valid_ndarray(arrays, ndim=3)
+        nb_arrays = len(arrays)
+        valid.valid_item(
+            correct_curvature, allowed_types=bool, name="correct_curvature"
+        )
+        valid.valid_item(debugging, allowed_types=bool, name="debugging")
+        if isinstance(fill_value, Real):
+            fill_value = (fill_value,) * nb_arrays
+        valid.valid_container(
+            fill_value,
+            container_types=(tuple, list, np.ndarray),
+            length=nb_arrays,
+            item_types=Real,
+            name="fill_value",
+        )
+
+        #####################################################
+        # recalculate the direct beam position with binning #
+        #####################################################
+        directbeam_y = int(
+            (self.direct_beam[0] - self.detector.roi[0]) / self.detector.binning[1]
+        )
+        # vertical
+        directbeam_x = int(
+            (self.direct_beam[1] - self.detector.roi[2]) / self.detector.binning[2]
+        )
+        # horizontal
+        print(
+            "\nDirect beam for the ROI and binning (y, x):", directbeam_y, directbeam_x
+        )
+
+        #######################################
+        # interpolate the diffraction pattern #
+        #######################################
+        if correct_curvature:
+            arrays, q_values = self.transformation_cdi_ewald(
+                arrays=arrays,
+                direct_beam=(directbeam_y, directbeam_x),
+                cdi_angle=cdi_angle,
+                fill_value=fill_value,
+            )
+        else:
+            arrays, q_values = self.transformation_cdi(
+                arrays=arrays,
+                direct_beam=(directbeam_y, directbeam_x),
+                cdi_angle=cdi_angle,
+                fill_value=fill_value,
+                debugging=debugging,
+            )
+        return arrays, q_values, (directbeam_y, directbeam_x)
+
     def ortho_directspace(
         self,
         arrays,
@@ -1204,11 +1539,12 @@ class Setup:
         ######################################################################
         # calculate the transformation matrix based on the beamline geometry #
         ######################################################################
-        transfer_matrix = self.transformation_matrix(
+        transfer_matrix, _ = self.transformation_bcdi(
             array_shape=input_shape,
             tilt_angle=tilt,
             pixel_x=pixel_x,
             pixel_y=pixel_y,
+            direct_space=True,
             verbose=verbose,
         )
 
@@ -1219,7 +1555,7 @@ class Setup:
         rotation_matrix = util.rotation_matrix_3d(
             axis_to_align=reference_axis, reference_axis=q_com / np.linalg.norm(q_com)
         )
-        # rotation_matrix = np.identity(3)
+
         ################################################
         # calculate the full transfer matrix including #
         # the rotation into the crystal frame          #
@@ -1279,8 +1615,6 @@ class Setup:
         nx_output += 10
         ny_output += 10
         nz_output += 10
-        del pos_along_x, pos_along_y, pos_along_z
-        gc.collect()
 
         #########################################
         # calculate the interpolation positions #
@@ -1313,8 +1647,6 @@ class Setup:
             + transfer_imatrix[2, 1] * myy
             + transfer_imatrix[2, 2] * myz
         )
-        del myx, myy, myz
-        gc.collect()
 
         ######################
         # interpolate arrays #
@@ -1515,7 +1847,7 @@ class Setup:
         ##########################################################
         # calculate the transformation matrix (the unit is 1/nm) #
         ##########################################################
-        transfer_matrix, q_offset = self.transformation_matrix(
+        transfer_matrix, q_offset = self.transformation_bcdi(
             array_shape=(nbz, nby, nbx),
             tilt_angle=self.tilt_angle,
             direct_space=False,
@@ -1577,7 +1909,6 @@ class Setup:
             #######################################################################
             # the center of mass of the diffraction
             # should be in the center of the array!
-            # TODO: implement any offset of the center of mass
             q_along_z_com = (
                 q_along_z[nbz // 2, nby // 2, nbx // 2] + q_offset[2]
             )  # q_offset in the order xyz
@@ -1650,9 +1981,6 @@ class Setup:
                     f" {dq_along_x:.5f} 1/nm)"
                 )
 
-        del q_along_x, q_along_y, q_along_z, myx, myy, myz
-        gc.collect()
-
         ##########################################################
         # crop the output shape in order to fit FFT requirements #
         ##########################################################
@@ -1674,16 +2002,12 @@ class Setup:
         # qx downstream, qz vertical up, qy outboard
         # this assumes that the center of mass of the diffraction pattern
         # was at the center of the array
-        # TODO : correct this if the diffraction pattern is not centered
-        qx = (
-            np.arange(-nz_output // 2, nz_output // 2, 1) * dq_along_z
-        )  # along z downstream
-        qz = (
-            np.arange(-ny_output // 2, ny_output // 2, 1) * dq_along_y
-        )  # along y vertical up
-        qy = (
-            np.arange(-nx_output // 2, nx_output // 2, 1) * dq_along_x
-        )  # along x outboard
+        qx = np.arange(-nz_output // 2, nz_output // 2, 1) * dq_along_z
+        # along z downstream
+        qz = np.arange(-ny_output // 2, ny_output // 2, 1) * dq_along_y
+        # along y vertical up
+        qy = np.arange(-nx_output // 2, nx_output // 2, 1) * dq_along_x
+        # along x outboard
 
         myz, myy, myx = np.meshgrid(qx, qz, qy, indexing="ij")
 
@@ -1708,8 +2032,6 @@ class Setup:
             + transfer_imatrix[2, 1] * myy
             + transfer_imatrix[2, 2] * myz
         )
-        del myx, myy, myz
-        gc.collect()
 
         ######################
         # interpolate arrays #
@@ -1808,11 +2130,12 @@ class Setup:
             name="array_shape",
         )
 
-        ortho_matrix = self.transformation_matrix(
+        ortho_matrix, _ = self.transformation_bcdi(
             array_shape=array_shape,
             tilt_angle=tilt_angle,
             pixel_x=pixel_x,
             pixel_y=pixel_y,
+            direct_space=True,
             verbose=verbose,
         )
         # ortho_matrix is the transformation matrix
@@ -1838,8 +2161,8 @@ class Setup:
         )
         return new_z, new_y, new_x
 
-    def transformation_matrix(
-        self, array_shape, tilt_angle, pixel_x, pixel_y, direct_space=True, verbose=True
+    def transformation_bcdi(
+        self, array_shape, tilt_angle, pixel_x, pixel_y, direct_space, verbose=True
     ):
         """
         Calculate the transformation matrix from detector frame to laboratory frame.
@@ -1855,7 +2178,6 @@ class Setup:
          direct space
         :param verbose: True to have printed comments
         :return:
-
          - the transformation matrix from the detector frame to the laboratory frame
          - the q offset (3D vector) if direct_space is False.
 
@@ -1905,11 +2227,218 @@ class Setup:
             mymatrix[:, 0] = array_shape[2] * mymatrix[:, 0]
             mymatrix[:, 1] = array_shape[1] * mymatrix[:, 1]
             mymatrix[:, 2] = array_shape[0] * mymatrix[:, 2]
-            return 2 * np.pi * np.linalg.inv(mymatrix).transpose()
+            return 2 * np.pi * np.linalg.inv(mymatrix).transpose(), None
 
         # reciprocal length scale in  1/nm
         return mymatrix, q_offset
 
+    def transformation_cdi(self, arrays, direct_beam, cdi_angle, fill_value, debugging):
+        """
+        Calculate the transformation matrix from detector frame to laboratory frame.
+
+        For the transformation in direct space, the length scale is in nm,
+        for the transformation in reciprocal space, it is in 1/nm.
+
+        :param arrays: tuple of 3D arrays of the same shape (e.g.: reciprocal space
+         diffraction pattern and mask), in the detector frame
+        :param direct_beam: tuple of 2 integers, position of the direction beam (V, H)
+        :param cdi_angle: 1D array of measurement angles in degrees
+        :param fill_value: tuple of real numbers (np.nan allowed), fill_value parameter
+         for the RegularGridInterpolator, same length as the number of arrays
+        :param debugging: bool, True to see more plots
+        :return:
+
+         - a tuple of arrays interpolated on an orthogonal grid (same length as the
+           number of input arrays)
+         - a tuple of three 1D arrays for the q values (qx, qz, qy) where qx is
+           downstream, qz is vertical up and qy is outboard.
+
+        """
+        #########################
+        # check some parameters #
+        #########################
+        valid.valid_ndarray(arrays, ndim=3)
+        valid.valid_container(
+            direct_beam,
+            container_types=(tuple, list),
+            length=2,
+            item_types=int,
+            name="direct_beam",
+        )
+        valid.valid_1d_array(
+            cdi_angle,
+            allowed_types=Real,
+            allow_none=False,
+            name="cdi_angle",
+        )
+        valid.valid_container(
+            fill_value,
+            container_types=(tuple, list),
+            length=2,
+            item_types=Real,
+            name="fill_value",
+        )
+        valid.valid_item(debugging, allowed_types=bool, name="debugging")
+
+        #########################
+        # convert lengths to nm #
+        #########################
+        wavelength = self.wavelength * 1e9
+        distance = self.distance * 1e9
+        pixel_x = self.detector.pixelsize_x * 1e9
+        # binned pixel size in the horizontal direction
+        pixel_y = self.detector.pixelsize_y * 1e9
+        # binned pixel size in the vertical direction
+        lambdaz = wavelength * distance
+
+        _, nby, nbx = arrays[0].shape
+        directbeam_y, directbeam_x = direct_beam
+        # calculate the number of voxels available to accomodate the gridded data
+        # directbeam_x and directbeam_y already are already taking into account
+        # the ROI and binning
+        numx = 2 * max(directbeam_x, nbx - directbeam_x)
+        # number of interpolated voxels in the plane perpendicular
+        # to the rotation axis. It will accomodate the full data range.
+        numy = nby  # no change of the voxel numbers along the rotation axis
+        print("\nData shape after regridding:", numx, numy, numx)
+
+        # update the direct beam position due to an eventual padding along X
+        if nbx - directbeam_x < directbeam_x:
+            pivot = directbeam_x
+        else:  # padding to the left along x, need to correct the pivot position
+            pivot = nbx - directbeam_x
+
+        dqx = 2 * np.pi / lambdaz * pixel_x
+        # in 1/nm, downstream, pixel_x is the binned pixel size
+        dqz = 2 * np.pi / lambdaz * pixel_y
+        # in 1/nm, vertical up, pixel_y is the binned pixel size
+        dqy = 2 * np.pi / lambdaz * pixel_x
+        # in 1/nm, outboard, pixel_x is the binned pixel size
+
+        ##########################################
+        # calculation of q based on P10 geometry #
+        ##########################################
+        qx = np.arange(-directbeam_x, -directbeam_x + numx, 1) * dqx
+        # downstream, same direction as detector X rotated by +90deg
+        qz = np.arange(directbeam_y - numy, directbeam_y, 1) * dqz
+        # vertical up opposite to detector Y
+        qy = np.arange(directbeam_x - numx, directbeam_x, 1) * dqy
+        # outboard opposite to detector X
+        print(
+            "q spacing for the interpolation (z,y,x) = "
+            f"({dqx:.6f}, {dqz:.6f},{dqy:.6f}) (1/nm)"
+        )
+
+        ##############################################################
+        # loop over 2D slices perpendicular to the rotation axis     #
+        # slower than doing a 3D interpolation but needs less memory #
+        ##############################################################
+        # find the corresponding polar coordinates of a cartesian 2D grid
+        # perpendicular to the rotation axis
+        interp_angle, interp_radius = self._beamline.cartesian2polar(
+            nb_pixels=numx,
+            pivot=pivot,
+            offset_angle=cdi_angle.min(),
+            debugging=debugging,
+        )
+
+        #################################################
+        # Interpolate the data onto a cartesian 3D grid #
+        #################################################
+        output_arrays = []
+        comment = ("data", "mask")
+        for idx, array in enumerate(arrays):
+            ortho_array = self.grid_cylindrical(
+                array=array,
+                rotation_angle=cdi_angle,
+                direct_beam=directbeam_x,
+                interp_angle=interp_angle,
+                interp_radius=interp_radius,
+                fill_value=fill_value[idx],
+                comment=comment[idx],
+            )
+            output_arrays.append(ortho_array)
+
+        return output_arrays, (qx, qz, qy)
+
+    def transformation_cdi_ewald(self, arrays, direct_beam, cdi_angle, fill_value):
+        """
+        Interpolate forward CDI data considering the curvature of the Ewald sphere.
+
+        :param arrays: tuple of 3D arrays of the same shape (e.g.: reciprocal space
+         diffraction pattern and mask), in the detector frame
+         :param direct_beam: tuple of 2 integers, position of the direction beam (V, H)
+        :param cdi_angle: 1D array of measurement angles in degrees
+        :param fill_value: tuple of real numbers (np.nan allowed), fill_value parameter
+         for the RegularGridInterpolator, same length as the number of arrays
+        :return:
+        """
+        nbz, nby, nbx = arrays[0].shape
+        _, directbeam_x = direct_beam
+        # calculate the number of voxels available to accomodate the gridded data
+        # directbeam_x and directbeam_y already are already taking into account
+        # the ROI and binning
+        numx = 2 * max(directbeam_x, nbx - directbeam_x)
+        # number of interpolated voxels in the plane perpendicular
+        # to the rotation axis. It will accomodate the full data range.
+        numy = nby  # no change of the voxel numbers along the rotation axis
+        print("\nData shape after regridding:", numx, numy, numx)
+
+        # calculate exact q values for each voxel of the 3D dataset
+        old_qx, old_qz, old_qy = self._beamline.ewald_curvature_saxs(
+            wavelength=self.wavelength * 1e9,
+            pixelsize_x=self.detector.pixelsize_x * 1e9,
+            pixelsize_y=self.detector.pixelsize_y * 1e9,
+            distance=self.distance * 1e9,
+            array_shape=(nbz, nby, nbx),
+            cdi_angle=cdi_angle,
+            direct_beam=direct_beam,
+        )
+
+        # create the grid for interpolation
+        qx = np.linspace(
+            old_qx.min(), old_qx.max(), numx, endpoint=False
+        )  # z downstream
+        qz = np.linspace(
+            old_qz.min(), old_qz.max(), numy, endpoint=False
+        )  # y vertical up
+        qy = np.linspace(old_qy.min(), old_qy.max(), numx, endpoint=False)  # x outboard
+
+        new_qx, new_qz, new_qy = np.meshgrid(qx, qz, qy, indexing="ij")
+
+        ###########################################################
+        # interpolate the data onto the new points using griddata #
+        # (the original grid is not regular, very slow)           #
+        ###########################################################
+        print("Interpolating the data using griddata, will take time...")
+        output_arrays = []
+        for idx, array in enumerate(arrays):
+            # convert array type to float,
+            # for integers the interpolation can lead to artefacts
+            array = array.astype(float)
+            ortho_array = griddata(
+                np.array(
+                    [
+                        np.ndarray.flatten(old_qx),
+                        np.ndarray.flatten(old_qz),
+                        np.ndarray.flatten(old_qy),
+                    ]
+                ).T,
+                np.ndarray.flatten(array),
+                np.array(
+                    [
+                        np.ndarray.flatten(new_qx),
+                        np.ndarray.flatten(new_qz),
+                        np.ndarray.flatten(new_qy),
+                    ]
+                ).T,
+                method="linear",
+                fill_value=fill_value[idx],
+            )
+            ortho_array = ortho_array.reshape((numx, numy, numx))
+            output_arrays.append(ortho_array)
+        return output_arrays, (qx, qz, qy)
+
     def voxel_sizes(self, array_shape, tilt_angle, pixel_x, pixel_y, verbose=False):
         """
         Calculate the direct space voxel sizes in the laboratory frame.
@@ -1933,12 +2462,12 @@ class Setup:
             name="array_shape",
         )
 
-        transfer_matrix = self.transformation_matrix(
+        transfer_matrix, _ = self.transformation_bcdi(
             array_shape=array_shape,
             tilt_angle=tilt_angle,
-            direct_space=True,
             pixel_x=pixel_x,
             pixel_y=pixel_y,
+            direct_space=True,
             verbose=verbose,
         )
         # transfer_matrix is the transformation matrix
