@@ -19,110 +19,14 @@ import matplotlib.pyplot as plt
 from scipy.interpolate import RegularGridInterpolator
 from scipy.ndimage.measurements import center_of_mass
 from scipy.signal import convolve
-from scipy.stats import multivariate_normal
-from scipy.stats import norm, pearsonr
+from scipy.stats import multivariate_normal, norm
 from skimage.restoration import unwrap_phase
+
 from ..graph import graph_utils as gu
-from ..preprocessing.preprocessing_utils import wrap
-from ..utils import image_registration as reg
 from ..utils import utilities as util
 from ..utils import validation as valid
 
 
-def align_obj(
-    reference_obj,
-    obj,
-    method="modulus",
-    support_threshold=None,
-    precision=1000,
-    debugging=False,
-):
-    """
-    Align two arrays using dft registration and subpixel shift.
-
-    :param reference_obj: 3D array, reference complex object
-    :param obj: 3D array, complex density to average with
-    :param method: 'modulus', 'support' or 'skip'. Object to use for the determination
-     of the shift. If 'support', the parameter 'support_threshold' must also be
-     provided since the binary support is defined by thresholding the normalized
-     modulus.
-    :param support_threshold: all points where the normalized modulus is larger than
-     this value will be set to 1 in the support.
-    :param precision: precision for the DFT registration in 1/pixel
-    :param debugging: set to True to see plots
-    :type debugging: bool
-    :return: the aligned array
-    """
-    valid.valid_ndarray(arrays=(obj, reference_obj), ndim=3, fix_shape=False)
-    if obj.shape != reference_obj.shape:
-        print(
-            "reference_obj and obj do not have the same shape\n",
-            reference_obj.shape,
-            obj.shape,
-            "crop/pad obj",
-        )
-        obj = util.crop_pad(array=obj, output_shape=reference_obj.shape)
-
-    # calculate the shift between the two arrays
-    if method == "modulus":
-        shiftz, shifty, shiftx = reg.getimageregistration(
-            abs(reference_obj), abs(obj), precision=precision
-        )
-    elif method == "support":
-        ref_support = np.zeros(reference_obj.shape)
-        ref_support[
-            abs(reference_obj) > support_threshold * abs(reference_obj).max()
-        ] = 1
-        support = np.zeros(reference_obj.shape)
-        support[abs(obj) > support_threshold * abs(obj).max()] = 1
-        shiftz, shifty, shiftx = reg.getimageregistration(
-            ref_support, support, precision=precision
-        )
-        if debugging:
-            gu.multislices_plot(
-                abs(ref_support), sum_frames=False, title="Reference support"
-            )
-            gu.multislices_plot(
-                abs(support), sum_frames=False, title="Support before alignement"
-            )
-        del ref_support, support
-    else:  # 'skip'
-        print("\nSkipping alignment")
-        print(
-            "\tPearson correlation coefficient = {0:.3f}".format(
-                pearsonr(
-                    np.ndarray.flatten(abs(reference_obj)), np.ndarray.flatten(abs(obj))
-                )[0]
-            )
-        )
-        return obj
-
-    # align obj using subpixel shift
-    new_obj = reg.subpixel_shift(obj, shiftz, shifty, shiftx)  # keep the complex output
-    print(
-        "\tShift calculated from dft registration: (",
-        str("{:.2f}".format(shiftz)),
-        ",",
-        str("{:.2f}".format(shifty)),
-        ",",
-        str("{:.2f}".format(shiftx)),
-        ") pixels",
-    )
-    print(
-        "\tPearson correlation coefficient = {0:.3f}".format(
-            pearsonr(
-                np.ndarray.flatten(abs(reference_obj)), np.ndarray.flatten(abs(new_obj))
-            )[0]
-        )
-    )
-    if debugging:
-        gu.multislices_plot(
-            abs(reference_obj), sum_frames=True, title="Reference object"
-        )
-        gu.multislices_plot(abs(new_obj), sum_frames=True, title="Aligned object")
-    return new_obj
-
-
 def apodize(amp, phase, initial_shape, window_type, debugging=False, **kwargs):
     """
     Apodize the complex array based on the window of the same shape.
@@ -256,204 +160,6 @@ def apodize(amp, phase, initial_shape, window_type, debugging=False, **kwargs):
     return abs(myobj), np.angle(myobj)
 
 
-def average_obj(
-    avg_obj,
-    ref_obj,
-    obj,
-    support_threshold=0.25,
-    correlation_threshold=0.90,
-    aligning_option="dft",
-    width_z=None,
-    width_y=None,
-    width_x=None,
-    method="reciprocal_space",
-    debugging=False,
-    **kwargs,
-):
-    """
-    Average two reconstructions after aligning it.
-
-    Alignment is processed only if their cross-correlation is larger than the parameter
-    correlation_threshold.
-
-    :param avg_obj: 3D array, average complex density
-    :param ref_obj: 3D array, reference complex object
-    :param obj: 3D array, complex density to average with
-    :param support_threshold: for support definition
-    :param correlation_threshold: minimum correlation between two dataset to average
-     them
-    :param aligning_option: 'com' for center of mass, 'dft' for dft registration and
-     subpixel shift
-    :param width_z: size of the area to plot in z (axis 0), centered on the middle of
-     the initial array
-    :param width_y: size of the area to plot in y (axis 1), centered on the middle of
-     the initial array
-    :param width_x: size of the area to plot in x (axis 2), centered on the middle of
-     the initial array
-    :param method: 'real_space' or 'reciprocal_space', in which space the average will
-     be performed
-    :param debugging: set to True to see plots
-    :type debugging: bool
-    :param kwargs:
-     - 'reciprocal_space': True if the object is in reciprocal space
-     - 'is_orthogonal': True if the data is in an orthonormal frame. Used for defining
-       default plot labels.
-
-    :return: the average complex density
-    """
-    valid.valid_ndarray(arrays=(obj, avg_obj, ref_obj), ndim=3)
-    # check and load kwargs
-    valid.valid_kwargs(
-        kwargs=kwargs,
-        allowed_kwargs={"reciprocal_space", "is_orthogonal"},
-        name="postprocessing_utils.average_obj",
-    )
-    reciprocal_space = kwargs.get("reciprocal_space", False)
-    is_orthogonal = kwargs.get("is_orthogonal", False)
-
-    nbz, nby, nbx = obj.shape
-    avg_flag = 0
-    if avg_obj.sum() == 0:
-        avg_obj = ref_obj
-        if debugging:
-            gu.multislices_plot(
-                abs(avg_obj),
-                width_z=width_z,
-                width_y=width_y,
-                width_x=width_x,
-                plot_colorbar=True,
-                sum_frames=True,
-                title="Reference object",
-                reciprocal_space=reciprocal_space,
-                is_orthogonal=is_orthogonal,
-            )
-    else:
-        myref_support = np.zeros((nbz, nby, nbx))
-        myref_support[abs(ref_obj) > support_threshold * abs(ref_obj).max()] = 1
-        my_support = np.zeros((nbz, nby, nbx))
-        my_support[abs(obj) > support_threshold * abs(obj).max()] = 1
-        avg_piz, avg_piy, avg_pix = center_of_mass(abs(myref_support))
-        piz, piy, pix = center_of_mass(abs(my_support))
-        offset_z = avg_piz - piz
-        offset_y = avg_piy - piy
-        offset_x = avg_pix - pix
-        print(
-            "center of mass offset with reference object: (",
-            str("{:.2f}".format(offset_z)),
-            ",",
-            str("{:.2f}".format(offset_y)),
-            ",",
-            str("{:.2f}".format(offset_x)),
-            ") pixels",
-        )
-        if aligning_option == "com":
-            # re-sample data on a new grid based on COM shift of support
-            old_z = np.arange(-nbz // 2, nbz // 2)
-            old_y = np.arange(-nby // 2, nby // 2)
-            old_x = np.arange(-nbx // 2, nbx // 2)
-            myz, myy, myx = np.meshgrid(old_z, old_y, old_x, indexing="ij")
-            new_z = myz + offset_z
-            new_y = myy + offset_y
-            new_x = myx + offset_x
-            del myx, myy, myz
-            rgi = RegularGridInterpolator(
-                (old_z, old_y, old_x),
-                obj,
-                method="linear",
-                bounds_error=False,
-                fill_value=0,
-            )
-            new_obj = rgi(
-                np.concatenate(
-                    (
-                        new_z.reshape((1, new_z.size)),
-                        new_y.reshape((1, new_z.size)),
-                        new_x.reshape((1, new_z.size)),
-                    )
-                ).transpose()
-            )
-            new_obj = new_obj.reshape((nbz, nby, nbx)).astype(obj.dtype)
-        else:
-            # dft registration and subpixel shift (see Matlab code)
-            shiftz, shifty, shiftx = reg.getimageregistration(
-                abs(ref_obj), abs(obj), precision=1000
-            )
-            new_obj = reg.subpixel_shift(
-                obj, shiftz, shifty, shiftx
-            )  # keep the complex output here
-            print(
-                "Shift calculated from dft registration: (",
-                str("{:.2f}".format(shiftz)),
-                ",",
-                str("{:.2f}".format(shifty)),
-                ",",
-                str("{:.2f}".format(shiftx)),
-                ") pixels",
-            )
-
-        new_obj = new_obj / abs(new_obj).max()  # renormalize
-
-        correlation = pearsonr(
-            np.ndarray.flatten(abs(ref_obj[np.nonzero(myref_support)])),
-            np.ndarray.flatten(abs(new_obj[np.nonzero(myref_support)])),
-        )[0]
-
-        if correlation < correlation_threshold:
-            print(
-                "pearson cross-correlation=",
-                correlation,
-                "too low, skip this reconstruction",
-            )
-        else:
-            print(
-                "pearson-correlation=",
-                correlation,
-                ", average with this reconstruction",
-            )
-
-            if debugging:
-                myfig, _, _ = gu.multislices_plot(
-                    abs(new_obj),
-                    width_z=width_z,
-                    width_y=width_y,
-                    width_x=width_x,
-                    sum_frames=True,
-                    plot_colorbar=True,
-                    title="Aligned object",
-                    reciprocal_space=reciprocal_space,
-                    is_orthogonal=is_orthogonal,
-                )
-                myfig.text(
-                    0.60,
-                    0.30,
-                    "pearson-correlation = " + str("{:.4f}".format(correlation)),
-                    size=20,
-                )
-
-            if method == "real_space":
-                avg_obj = avg_obj + new_obj
-            elif method == "reciprocal_space":
-                avg_obj = ifftn(fftn(avg_obj) + fftn(obj))
-            else:
-                raise ValueError('method should be "real_space" or "reciprocal_space"')
-            avg_flag = 1
-
-        if debugging:
-            gu.multislices_plot(
-                abs(avg_obj),
-                plot_colorbar=True,
-                width_z=width_z,
-                width_y=width_y,
-                width_x=width_x,
-                sum_frames=True,
-                title="New averaged object",
-                reciprocal_space=reciprocal_space,
-                is_orthogonal=is_orthogonal,
-            )
-
-    return avg_obj, avg_flag
-
-
 def blackman_window(shape, normalization=1):
     """
     Create a 3d Blackman window based on shape.
@@ -1026,7 +732,7 @@ def find_crop_center(array_shape, crop_shape, pivot):
         name="pivot",
     )
     crop_center = np.empty(ndim)
-    for idx, dim in enumerate(range(ndim)):
+    for idx, _ in enumerate(range(ndim)):
         if max(0, pivot[idx] - crop_shape[idx] // 2) == 0:
             # not enough range on this side of the com
             crop_center[idx] = crop_shape[idx] // 2
@@ -1040,7 +746,7 @@ def find_crop_center(array_shape, crop_shape, pivot):
             else:
                 crop_center[idx] = pivot[idx]
 
-    crop_center = list(map(lambda x: int(x), crop_center))
+    crop_center = list(map(int, crop_center))
     return crop_center
 
 
@@ -1429,7 +1135,7 @@ def get_strain(
             temp_phase = np.copy(phase)
             temp_phase = temp_phase + offset
             # wrap again the offseted phase
-            temp_phase = wrap(
+            temp_phase = util.wrap(
                 obj=temp_phase, start_angle=-extent_phase / 2, range_angle=extent_phase
             )
         else:  # no need to copy the phase, offset = 0
@@ -1649,7 +1355,7 @@ def ortho_modes(array_stack, nb_mode=None, method="eig", verbose=False):
         )  # the number of eigenvalues is nb_arrays
     elif method == "svd":  # Singular value decomposition
         my_matrix = np.reshape(array_stack, (nb_arrays, array_size))
-        eigenvectors, eigenvalues, vh = scipy.linalg.svd(
+        eigenvectors, eigenvalues, _ = scipy.linalg.svd(
             my_matrix, full_matrices=False, compute_uv=True
         )
         # my_matrix = eigenvectors x S x Vh,
@@ -2474,27 +2180,25 @@ def unwrap(obj, support_threshold, seed=0, debugging=True, **kwargs):
     ] = 0  # 0 is a valid entry for ma.masked_array
     phase_wrapped = ma.masked_array(np.angle(obj), mask=unwrap_support)
 
-    if debugging:
-        if ndim == 3:
-            gu.multislices_plot(
-                phase_wrapped.data,
-                plot_colorbar=True,
-                title="Object before unwrapping",
-                reciprocal_space=reciprocal_space,
-                is_orthogonal=is_orthogonal,
-            )
+    if debugging and ndim == 3:
+        gu.multislices_plot(
+            phase_wrapped.data,
+            plot_colorbar=True,
+            title="Object before unwrapping",
+            reciprocal_space=reciprocal_space,
+            is_orthogonal=is_orthogonal,
+        )
 
     phase_unwrapped = unwrap_phase(phase_wrapped, wrap_around=False, seed=seed).data
     phase_unwrapped[np.nonzero(unwrap_support)] = 0
-    if debugging:
-        if ndim == 3:
-            gu.multislices_plot(
-                phase_unwrapped,
-                plot_colorbar=True,
-                title="Object after unwrapping",
-                reciprocal_space=reciprocal_space,
-                is_orthogonal=is_orthogonal,
-            )
+    if debugging and ndim == 3:
+        gu.multislices_plot(
+            phase_unwrapped,
+            plot_colorbar=True,
+            title="Object after unwrapping",
+            reciprocal_space=reciprocal_space,
+            is_orthogonal=is_orthogonal,
+        )
 
     extent_phase = np.ceil(phase_unwrapped.max() - phase_unwrapped.min())
     return phase_unwrapped, extent_phase
