@@ -7,11 +7,6 @@
 #       authors:
 #         Jerome Carnis, carnis_jerome@yahoo.fr
 
-helptext = """
-Open mesh scans and plot interactively the integrated intensity vs. motor positions for
-a user-defined region of interest.
-"""
-
 try:
     import hdf5plugin  # for P10, should be imported before h5py or PyTables
 except ModuleNotFoundError:
@@ -24,10 +19,15 @@ import os
 import bcdi.graph.graph_utils as gu
 from bcdi.experiment.detector import create_detector
 from bcdi.experiment.setup import Setup
-import bcdi.preprocessing.preprocessing_utils as pru
+import bcdi.preprocessing.bcdi_utils as bu
 
 matplotlib.use("Qt5Agg")
 
+helptext = """
+Open mesh scans and plot interactively the integrated intensity vs. motor positions for
+a user-defined region of interest.
+"""
+
 scan = 26  # scan number as it appears in the folder name
 sample_name = "B10_syn_S5"  # without _ at the end
 root_folder = "D:/data/P10_Longfei/"
@@ -63,10 +63,9 @@ invert_yaxis = True  # True to inverse the vertical axis
 ###############################
 # beamline related parameters #
 ###############################
-beamline = (
-    "P10"  # name of the beamlisne, used for data loading and normalization by monitor
-)
-# supported beamlines: 'P10' only for now, see preprocessing_utils.get_motor_pos()
+beamline = "P10"
+# name of the beamlisne, used for data loading and normalization by monitor
+# supported beamlines: 'P10' only for now
 is_series = False  # specific to series measurement at P10
 specfile_name = ""
 # .spec for ID01, .fio for P10, alias_dict.txt for SIXS_2018,
@@ -324,7 +323,7 @@ if not (
 #############
 # load data #
 #############
-data, mask, monitor, frames_logical = pru.load_data(
+data, mask, monitor, frames_logical = setup.diffractometer.load_check_dataset(
     logfile=logfile,
     scan_number=scan,
     detector=detector,
@@ -344,10 +343,10 @@ data[data <= threshold] = 0
 ########################
 # load motor positions #
 ########################
-fast_positions = pru.get_motor_pos(
+fast_positions = setup.diffractometer.read_device(
     logfile=logfile, scan_number=scan, setup=setup, motor_name=fast_motor
 )
-slow_positions = pru.get_motor_pos(
+slow_positions = setup.diffractometer.read_device(
     logfile=logfile, scan_number=scan, setup=setup, motor_name=slow_motor
 )
 
