@@ -11,20 +11,20 @@ try:
     import hdf5plugin  # for P10, should be imported before h5py or PyTables
 except ModuleNotFoundError:
     pass
+from matplotlib import pyplot as plt
 import numpy as np
 from numpy.fft import fftn, fftshift
-from matplotlib import pyplot as plt
+from scipy.interpolate import interp1d
 from scipy.ndimage.measurements import center_of_mass
 import tkinter as tk
 from tkinter import filedialog
 import xrayutilities as xu
-from scipy.interpolate import interp1d
 import gc
 import sys
 import bcdi.graph.graph_utils as gu
 from bcdi.experiment.detector import create_detector
 from bcdi.experiment.setup import Setup
-import bcdi.preprocessing.preprocessing_utils as pru
+import bcdi.utils.image_registration as reg
 import bcdi.utils.utilities as util
 
 helptext = """
@@ -74,8 +74,6 @@ actuators = {}
 # e.g.  {'rocking_angle': 'actuator_1_3', 'detector': 'data_04', 'monitor': 'data_05'}
 is_series = False  # specific to series measurement at P10
 rocking_angle = "outofplane"  # "outofplane" or "inplane"
-follow_bragg = False  # only for energy scans, set to True if the detector
-# was also scanned to follow the Bragg peak
 specfile_name = "alignment"
 # .spec for ID01, .fio for P10, alias_dict.txt for SIXS_2018,
 # not used for CRISTAL and SIXS_2019
@@ -184,7 +182,6 @@ setup.init_paths(
     save_dir=save_dir,
     specfile_name=specfile_name,
     template_imagefile=template_imagefile,
-    create_savedir=True,
 )
 
 logfile = setup.create_logfile(
@@ -365,14 +362,11 @@ if simulation:
         eta, 0, 0, inplane_simu, outofplane_simu, delta=(0, 0, 0, 0, 0)
     )
 else:
-    qx, qz, qy, _ = pru.regrid(
+    qx, qz, qy, _ = setup.calc_qvalues_xrutils(
         logfile=logfile,
+        hxrd=hxrd,
         nb_frames=numz,
         scan_number=scan,
-        detector=detector,
-        setup=setup,
-        hxrd=hxrd,
-        follow_bragg=follow_bragg,
     )
 
 if debug:
@@ -491,11 +485,10 @@ if debug:
 
 if align_pattern:
     # align the reconstruction with the initial diffraction data
-    phased_fft, _ = pru.align_diffpattern(
+    phased_fft, _ = reg.align_diffpattern(
         reference_data=slice_2D,
         data=phased_fft,
-        method="registration",
-        combining_method="subpixel",
+        interpolation_method="subpixel",
     )
 
 plt.figure()
