@@ -17,7 +17,7 @@ from scipy.interpolate import interp1d
 from scipy.ndimage.measurements import center_of_mass
 from bcdi.experiment.detector import create_detector
 from bcdi.experiment.setup import Setup
-import bcdi.preprocessing.preprocessing_utils as pru
+import bcdi.preprocessing.bcdi_utils as bu
 import bcdi.graph.graph_utils as gu
 import bcdi.utils.utilities as util
 import bcdi.utils.validation as valid
@@ -35,14 +35,20 @@ scans = np.concatenate((scans, np.arange(1638, 1680 + 1, 3)))
 root_folder = "D:/data/P10_OER/data/"
 sample_name = "dewet2_2"  # list of sample names. If only one name is indicated,
 # it will be repeated to match the number of scans
+data_dir = None
+# leave None to use the beamline default. It will look for the data at this location
 save_dir = "D:/data/P10_OER/analysis/candidate_12/"
 # images will be saved here, leave it to None otherwise (default to root_folder)
-x_axis = []
-[x_axis.append(0.740) for _ in range(16)]
-[x_axis.append(0.80) for _ in range(10)]
-[x_axis.append(-0.05) for _ in range(15)]
-[x_axis.append(0.3) for _ in range(15)]
-[x_axis.append(0.8) for _ in range(15)]
+x_axis = [0.740 for _ in range(16)]
+for _ in range(10):
+    x_axis.append(0.80)
+for _ in range(15):
+    x_axis.append(-0.05)
+for _ in range(15):
+    x_axis.append(0.3)
+for _ in range(15):
+    x_axis.append(0.8)
+
 # values against which the Bragg peak center of mass evolution will be plotted,
 # leave [] otherwise
 x_label = "voltage (V)"  # label for the X axis in plots, leave '' otherwise
@@ -224,10 +230,9 @@ for scan_idx, scan_nb in enumerate(scans, start=1):
         scan_number=scan_nb,
         root_folder=root_folder,
         save_dir=save_dir,
-        verbose=True,
-        create_savedir=True,
         specfile_name=specfile_name,
         template_imagefile=template_imagefile,
+        data_dir=data_dir,
     )
 
     # override the saving directory, we want to save results at the same place
@@ -237,7 +242,7 @@ for scan_idx, scan_nb in enumerate(scans, start=1):
         scan_number=scan_nb, root_folder=root_folder, filename=detector.specfile
     )
 
-    data, mask, frames_logical, monitor = pru.load_bcdi_data(
+    data, mask, frames_logical, monitor = bu.load_bcdi_data(
         logfile=logfile,
         scan_number=scan_nb,
         detector=detector,
@@ -466,9 +471,9 @@ x_axis = np.asarray(x_axis)
 uniq_xaxis = np.unique(x_axis)
 mean_tilt = np.empty(len(uniq_xaxis))
 std_tilt = np.empty(len(uniq_xaxis))
-for idx in range(len(uniq_xaxis)):
-    mean_tilt[idx] = np.mean(tilt_com[x_axis == uniq_xaxis[idx]])
-    std_tilt[idx] = np.std(tilt_com[x_axis == uniq_xaxis[idx]])
+for idx, item in enumerate(uniq_xaxis):
+    mean_tilt[idx] = np.mean(tilt_com[x_axis == item])
+    std_tilt[idx] = np.std(tilt_com[x_axis == item])
 
 fig, (ax0, ax1, ax2) = plt.subplots(nrows=1, ncols=3, figsize=(12, 9))
 ax0.plot(scans, tilt_com, "-o")
@@ -504,9 +509,9 @@ if convert_to_q:
     q_com = np.asarray(q_com)
     mean_q = np.empty(len(uniq_xaxis))
     std_q = np.empty(len(uniq_xaxis))
-    for idx in range(len(uniq_xaxis)):
-        mean_q[idx] = np.mean(q_com[x_axis == uniq_xaxis[idx]])
-        std_q[idx] = np.std(q_com[x_axis == uniq_xaxis[idx]])
+    for idx, item in enumerate(uniq_xaxis):
+        mean_q[idx] = np.mean(q_com[x_axis == item])
+        std_q[idx] = np.std(q_com[x_axis == item])
 
     fig, (ax0, ax1, ax2) = plt.subplots(nrows=1, ncols=3, figsize=(12, 9))
     ax0.plot(scans, q_com, "-o")
