@@ -11,13 +11,9 @@ try:
     import hdf5plugin  # for P10, should be imported before h5py or PyTables
 except ModuleNotFoundError:
     pass
-import xrayutilities as xu
-import numpy as np
+import gc
 import matplotlib.pyplot as plt
-
-plt.switch_backend(
-    "Qt5Agg"
-)  # "Qt5Agg" or "Qt4Agg" depending on the version of Qt installer, bug with Tk
+import numpy as np
 import os
 import scipy.signal  # for medfilt2d
 from scipy.ndimage.measurements import center_of_mass
@@ -25,16 +21,15 @@ import sys
 from scipy.io import savemat
 import tkinter as tk
 from tkinter import filedialog
-import gc
+import xrayutilities as xu
 import bcdi.graph.graph_utils as gu
 from bcdi.experiment.detector import create_detector
 from bcdi.experiment.setup import Setup
 import bcdi.postprocessing.postprocessing_utils as pu
-import bcdi.preprocessing.preprocessing_utils as pru
+import bcdi.preprocessing.bcdi_utils as bu
 import bcdi.utils.utilities as util
 import bcdi.utils.validation as valid
 
-
 helptext = """
 Prepare experimental data for Bragg CDI phasing: crop/pad, center, mask, normalize and
 filter the data.
@@ -52,34 +47,35 @@ output files saved in:   /rootdir/S1/pynxraw/ or /rootdir/S1/pynx/ depending on
 'use_rawdata' option
 """
 
-scans = 76  # np.arange(1401, 1419+1, 3)  # scan number or list of scan numbers
+scans = 11  # np.arange(1401, 1419+1, 3)  # scan number or list of scan numbers
 # scans = np.concatenate((scans, np.arange(1147, 1195+1, 3)))
 # bad_indices = np.argwhere(scans == 738)
 # scans = np.delete(scans, bad_indices)
 
-root_folder = "C:/Users/carnisj/Documents/data/P10_Longfei_Nov2020/data/"
+root_folder = "C:/Users/Jerome/Documents/data/dataset_ID01/"
 # folder of the experiment, where all scans are stored
 save_dir = root_folder + "test/"  # images will be saved here, leave it to None
 # otherwise
-data_dirname = None  # leave None to use the beamline default,
-# '' empty string when there is no subfolder
-# (data directly in the scan folder), or a non-empty string for the subfolder name
-# (default to scan_folder/pynx/ or scan_folder/pynxraw/
-# depending on the setting of use_rawdata)
-sample_name = "B15_syn_S1_2"  # str or list of str of sample names
+data_dir = root_folder + "ici/"
+# leave None to use the beamline default. It will look for the data at this location
+sample_name = "S"  # str or list of str of sample names
 # (string in front of the scan number in the folder name).
 # If only one name is indicated, it will be repeated to match the number of scans.
 user_comment = ""  # string, should start with "_"
 debug = False  # set to True to see plots
 binning = (1, 2, 2)  # binning to apply to the data
 # (stacking dimension, detector vertical axis, detector horizontal axis)
+bin_during_loading = False  # True to bin during loading, require less memory
+frames_pattern = None
+# 1D array of int, of length data.shape[0]. If frames_pattern is 0 at index,
+# the frame at data[index] will be skipped, if 1 the frame will added to the stack.
+# Use this if you need to remove some frames and you know it in advance.
 ##############################
 # parameters used in masking #
 ##############################
 flag_interact = False  # True to interact with plots, False to close it automatically
-background_plot = (
-    "0.5"  # in level of grey in [0,1], 0 being dark. For visual comfort during masking
-)
+background_plot = "0.5"
+# in level of grey in [0,1], 0 being dark. For visual comfort during masking
 #########################################################
 # parameters related to data cropping/padding/centering #
 #########################################################
@@ -118,14 +114,10 @@ medfilt_order = 7  # for custom median filter,
 # parameters used when reloading processed data #
 #################################################
 reload_previous = False  # True to resume a previous masking (load data and mask)
-reload_orthogonal = (
-    False  # True if the reloaded data is already intepolated in an orthonormal frame
-)
-preprocessing_binning = (
-    1,
-    1,
-    1,
-)  # binning factors in each dimension of the binned data to be reloaded
+reload_orthogonal = False
+# True if the reloaded data is already intepolated in an orthonormal frame
+preprocessing_binning = (1, 1, 1)
+# binning factors in each dimension of the binned data to be reloaded
 ##################
 # saving options #
 ##################
@@ -133,20 +125,19 @@ save_rawdata = False  # save also the raw data when use_rawdata is False
 save_to_npz = True  # True to save the processed data in npz format
 save_to_mat = False  # True to save also in .mat format
 save_to_vti = False  # save the orthogonalized diffraction pattern to VTK file
-save_asint = (
-    False  # if True, the result will be saved as an array of integers (save space)
-)
+save_asint = False
+# if True, the result will be saved as an array of integers (save space)
 ######################################
 # define beamline related parameters #
 ######################################
-beamline = (
-    "P10"  # name of the beamline, used for data loading and normalization by monitor
-)
+beamline = "ID01"
+# name of the beamline, used for data loading and normalization by monitor
 # supported beamlines: 'ID01', 'SIXS_2018', 'SIXS_2019', 'CRISTAL', 'P10',
 # 'NANOMAX', '34ID'
 actuators = None  # {'rocking_angle': 'actuator_1_1'}
 # Optional dictionary that can be used to define the entries corresponding to
 # actuators in data files (useful at CRISTAL where the location of data keeps changing)
+# or to define a monitor different from "exp1" at ID01
 # e.g.  {'rocking_angle': 'actuator_1_3', 'detector': 'data_04', 'monitor': 'data_05'}
 is_series = True  # specific to series measurement at P10
 
@@ -155,27 +146,26 @@ custom_scan = False  # set it to True for a stack of images acquired without sca
 # there is no spec/log file available
 custom_images = [3]  # np.arange(11353, 11453, 1)
 # list of image numbers for the custom_scan, None otherwise
-custom_monitor = np.ones(
-    51
-)  # monitor values for normalization for the custom_scan, None otherwise
+custom_monitor = np.ones(51)
+# monitor values for normalization for the custom_scan, None otherwise
 
 rocking_angle = "outofplane"  # "outofplane" for a sample rotation around x outboard,
 # "inplane" for a sample rotation around y vertical up, "energy"
 
-follow_bragg = False  # only for energy scans, set to True if the detector
-# was also scanned to follow the Bragg peak
-specfile_name = ""
-# template for ID01: name of the spec file without '.spec'
-# template for SIXS: full path of the alias dictionnary or
-# None to use the one in the package folder
-# template for all other beamlines: ''
+specfile_name = "l5.spec"
+# template for ID01 and 34ID: name of the spec file if it is at the default location
+# (in root_folder) or full path to the spec file
+# template for SIXS: full path of the alias dictionnary or None to use the one in the
+# package folder
+# for P10, either None (if you are using the same directory structure as the beamline)
+# or the full path to the .fio file
+# template for all other beamlines: None
 ###############################
 # detector related parameters #
 ###############################
-detector = "Eiger4M"  # "Eiger2M", "Maxipix", "Eiger4M", "Merlin" or "Timepix"
-linearity_func = (
-    None  # lambda array_1d: (array_1d*(7.484e-22*array_1d**4 - 3.447e-16*array_1d**3 +
-)
+detector = "Maxipix"  # "Eiger2M", "Maxipix", "Eiger4M", "Merlin" or "Timepix"
+linearity_func = None
+# lambda array_1d: (array_1d*(7.484e-22*array_1d**4 - 3.447e-16*array_1d**3 +
 # 5.067e-11*array_1d**2 - 6.022e-07*array_1d + 0.889)) # MIR
 # np.divide(array_1d, (1-array_1d*1.3e-6))  # Sarah_1
 # (array_1d*(7.484e-22*array_1d**4 - 3.447e-16*array_1d**3 +
@@ -187,7 +177,7 @@ x_bragg = 1577  # horizontal pixel number of the Bragg peak,
 # can be used for the definition of the ROI
 y_bragg = 833  # vertical pixel number of the Bragg peak,
 # can be used for the definition of the ROI
-roi_detector = [y_bragg - 216, y_bragg + 216, x_bragg - 200, x_bragg + 200]  #
+roi_detector = None  # [y_bragg - 216, y_bragg + 216, x_bragg - 200, x_bragg + 200]  #
 # [Vstart, Vstop, Hstart, Hstop]
 # leave None to use the full detector.
 # Use with center_fft='skip' if you want this exact size.
@@ -199,10 +189,9 @@ photon_filter = "loading"  # 'loading' or 'postprocessing',
 background_file = None  # root_folder + 'background.npz'  # non empty file path or None
 hotpixels_file = None  # root_folder + 'hotpixels_cristal.npz'
 # root_folder + 'mask_merlin.npy'  # non empty file path or None
-flatfield_file = (
-    None  # root_folder + "flatfield_maxipix_8kev.npz"  # non empty file path or None
-)
-template_imagefile = "_master.h5"
+flatfield_file = None
+# root_folder + "flatfield_maxipix_8kev.npz"  # non empty file path or None
+template_imagefile = "data_mpx4_%05d.edf.gz"
 # template for ID01: 'data_mpx4_%05d.edf.gz' or 'align_eiger2M_%05d.edf.gz'
 # template for SIXS_2018: 'align.spec_ascan_mu_%05d.nxs'
 # template for SIXS_2019: 'spare_ascan_mu_%05d.nxs'
@@ -210,36 +199,25 @@ template_imagefile = "_master.h5"
 # template for P10: '_master.h5'
 # template for NANOMAX: '%06d.h5'
 # template for 34ID: 'Sample%dC_ES_data_51_256_256.npz'
-nb_pixel_x = None  # fix to declare a known detector but with less pixels
-# (e.g. one tile HS), leave None otherwise
-nb_pixel_y = None  # fix to declare a known detector but with less pixels
-# (e.g. one tile HS), leave None otherwise
 ################################################################################
 # define parameters below if you want to orthogonalize the data before phasing #
 ################################################################################
-use_rawdata = False  # False for using data gridded in laboratory frame
+use_rawdata = True  # False for using data gridded in laboratory frame
 # True for using data in detector frame
-interp_method = "linearization"  # 'xrayutilities' or 'linearization'
+interp_method = "xrayutilities"  # 'xrayutilities' or 'linearization'
 fill_value_mask = 0  # 0 (not masked) or 1 (masked).
 # It will define how the pixels outside of the data range are
 # processed during the interpolation. Because of the large number of masked pixels,
 # phase retrieval converges better if the pixels are not masked (0 intensity
 # imposed). The data is by default set to 0 outside of the defined range.
-beam_direction = (
-    1,
-    0,
-    0,
-)  # beam direction in the laboratory frame (downstream, vertical up, outboard)
-sample_offsets = (
-    0,
-    0,
-    90,
-    0,
-)  # tuple of offsets in degrees of the sample for each sample circle (outer first).
+beam_direction = (1, 0, 0)
+# beam direction in the laboratory frame (downstream, vertical up, outboard)
+sample_offsets = None
+# tuple of offsets in degrees of the sample for each sample circle (outer first).
 # convention: the sample offsets will be subtracted to the motor values.
 # Leave None if no offset.
-sdd = 1.84  # in m, sample to detector distance in m
-energy = 8170  # np.linspace(11100, 10900, num=51)  # x-ray energy in eV
+sdd = 0.50678  # in m, sample to detector distance in m
+energy = 9000  # np.linspace(11100, 10900, num=51)  # x-ray energy in eV
 custom_motors = None
 # {"mu": 0, "phi": -15.98, "chi": 90, "theta": 0, "delta": -0.5685, "gamma": 33.3147}
 # use this to declare motor positions if there is not log file, None otherwise
@@ -256,34 +234,26 @@ custom_motors = None
 # parameters when orthogonalizing the data before     #
 # phasing  using the linearized transformation matrix #
 ######################################################
-align_q = True  # used only when interp_method is 'linearization',
+align_q = False  # used only when interp_method is 'linearization',
 # if True it rotates the crystal to align q
 # along one axis of the array
 ref_axis_q = "y"  # q will be aligned along that axis
-outofplane_angle = (
-    42.6093  # detector angle in deg (rotation around x outboard, typically delta),
-)
+outofplane_angle = 42.6093
+# detector angle in deg (rotation around x outboard, typically delta),
+
 # corrected for the direct beam position. Leave None to use the uncorrected position.
-inplane_angle = (
-    -0.5783
-)  # detector angle in deg(rotation around y vertical up, typically gamma),
+inplane_angle = -0.5783
+# detector angle in deg(rotation around y vertical up, typically gamma),
 # corrected for the direct beam position. Leave None to use the uncorrected position.
 ################################################################################
 # parameters when orthogonalizing the data before phasing  using xrayutilities #
 ################################################################################
 # xrayutilities uses the xyz crystal frame: for incident angle = 0,
 # x is downstream, y outboard, and z vertical up
-sample_inplane = (
-    1,
-    0,
-    0,
-)
+sample_inplane = (1, 0, 0)
 # sample inplane reference direction along the beam at 0 angles in xrayutilities frame
-sample_outofplane = (
-    0,
-    0,
-    1,
-)  # surface normal of the sample at 0 angles in xrayutilities frame
+sample_outofplane = (0, 0, 1)
+# surface normal of the sample at 0 angles in xrayutilities frame
 offset_inplane = 0  # outer detector angle offset as determined by
 # xrayutilities area detector initialization
 cch1 = 208  # direct beam vertical position in the full unbinned detector for
@@ -452,12 +422,15 @@ valid_name = "bcdi_preprocess_BCDI"
 if isinstance(scans, int):
     scans = (scans,)
 
-if len(scans) > 1:
-    if center_fft not in ["crop_asymmetric_ZYX", "pad_Z", "pad_asymmetric_ZYX"]:
-        center_fft = "skip"
-        # avoid croping the detector plane XY while centering the Bragg peak
-        # otherwise outputs may have a different size,
-        # which will be problematic for combining or comparing them
+if len(scans) > 1 and center_fft not in [
+    "crop_asymmetric_ZYX",
+    "pad_Z",
+    "pad_asymmetric_ZYX",
+]:
+    center_fft = "skip"
+    # avoid croping the detector plane XY while centering the Bragg peak
+    # otherwise outputs may have a different size,
+    # which will be problematic for combining or comparing them
 if len(fix_size) != 0:
     print('"fix_size" parameter provided, roi_detector will be set to []')
     roi_detector = []
@@ -614,12 +587,10 @@ for scan_idx, scan_nb in enumerate(scans, start=1):
     setup.init_paths(
         sample_name=sample_name[scan_idx - 1],
         scan_number=scan_nb,
-        data_dirname=data_dirname,
+        data_dir=data_dir,
         root_folder=root_folder,
         save_dir=save_dir,
         save_dirname=save_dirname,
-        verbose=True,
-        create_savedir=create_savedir,
         specfile_name=specfile_name,
         template_imagefile=template_imagefile,
     )
@@ -643,7 +614,6 @@ for scan_idx, scan_nb in enumerate(scans, start=1):
                 logfile=logfile,
                 scan_number=scan_nb,
                 setup=setup,
-                follow_bragg=follow_bragg,
             )
             setup.tilt_angle = (tilt_angle[1:] - tilt_angle[0:-1]).mean()
             # override detector motor positions if the corrected values
@@ -744,7 +714,7 @@ for scan_idx, scan_nb in enumerate(scans, start=1):
                     ]  # along x outboard
                     del numz, numy, numx
         else:  # the data is in the detector frame
-            data, mask, frames_logical, monitor = pru.reload_bcdi_data(
+            data, mask, frames_logical, monitor = bu.reload_bcdi_data(
                 logfile=logfile,
                 scan_number=scan_nb,
                 data=data,
@@ -762,11 +732,13 @@ for scan_idx, scan_nb in enumerate(scans, start=1):
         hotpix_array = util.load_hotpixels(hotpixels_file)
         background = util.load_background(background_file)
 
-        data, mask, frames_logical, monitor = pru.load_bcdi_data(
+        data, mask, frames_logical, monitor = bu.load_bcdi_data(
             logfile=logfile,
             scan_number=scan_nb,
             detector=detector,
             setup=setup,
+            frames_pattern=frames_pattern,
+            bin_during_loading=bin_during_loading,
             flatfield=flatfield,
             hotpixels=hotpix_array,
             background=background,
@@ -878,7 +850,7 @@ for scan_idx, scan_nb in enumerate(scans, start=1):
                 # first two arguments in init_area are the direction of the detector,
                 # checked for ID01 and SIXS
 
-                data, mask, q_values, frames_logical = pru.grid_bcdi_xrayutil(
+                data, mask, q_values, frames_logical = bu.grid_bcdi_xrayutil(
                     data=data,
                     mask=mask,
                     scan_number=scan_nb,
@@ -887,7 +859,6 @@ for scan_idx, scan_nb in enumerate(scans, start=1):
                     setup=setup,
                     frames_logical=frames_logical,
                     hxrd=hxrd,
-                    follow_bragg=follow_bragg,
                     debugging=debug,
                 )
             else:  # 'linearization'
@@ -895,7 +866,7 @@ for scan_idx, scan_nb in enumerate(scans, start=1):
                 # (qx downstream, qy outboard, qz vertical up)
                 # for reference_axis, the frame is z downstream, y vertical up,
                 # x outboard but the order must be x,y,z
-                data, mask, q_values = pru.grid_bcdi_labframe(
+                data, mask, q_values = bu.grid_bcdi_labframe(
                     data=data,
                     mask=mask,
                     detector=detector,
@@ -903,7 +874,6 @@ for scan_idx, scan_nb in enumerate(scans, start=1):
                     align_q=align_q,
                     reference_axis=axis_to_array_xyz[ref_axis_q],
                     debugging=debug,
-                    follow_bragg=follow_bragg,
                     fill_value=(0, fill_value_mask),
                 )
             nz, ny, nx = data.shape
@@ -959,7 +929,7 @@ for scan_idx, scan_nb in enumerate(scans, start=1):
     ########################
     # crop/pad/center data #
     ########################
-    data, mask, pad_width, q_values, frames_logical = pru.center_fft(
+    data, mask, pad_width, q_values, frames_logical = bu.center_fft(
         data=data,
         mask=mask,
         detector=detector,
@@ -1269,7 +1239,7 @@ for scan_idx, scan_nb in enumerate(scans, start=1):
         for idx in range(
             pad_width[0], nz - pad_width[1]
         ):  # filter only frames whith data (not padded)
-            data[idx, :, :], processed_pix, mask[idx, :, :] = pru.mean_filter(
+            data[idx, :, :], processed_pix, mask[idx, :, :] = util.mean_filter(
                 data=data[idx, :, :],
                 nb_neighbours=medfilt_order,
                 mask=mask[idx, :, :],
