@@ -11,12 +11,9 @@ try:
     import hdf5plugin  # for P10, should be imported before h5py or PyTables
 except ModuleNotFoundError:
     pass
-import numpy as np
+import gc
 import matplotlib.pyplot as plt
-
-plt.switch_backend(
-    "Qt5Agg"
-)  # "Qt5Agg" or "Qt4Agg" depending on the version of Qt installer, bug with Tk
+import numpy as np
 import os
 import scipy.signal  # for medfilt2d
 from scipy.ndimage.measurements import center_of_mass
@@ -24,12 +21,11 @@ import sys
 from scipy.io import savemat
 import tkinter as tk
 from tkinter import filedialog
-import gc
 import bcdi.graph.graph_utils as gu
 from bcdi.experiment.detector import create_detector
 from bcdi.experiment.setup import Setup
 import bcdi.utils.utilities as util
-import bcdi.preprocessing.preprocessing_utils as pru
+import bcdi.preprocessing.cdi_utils as cdi
 import bcdi.utils.validation as valid
 
 helptext = """
@@ -46,18 +42,24 @@ Output files are saved in:
 """
 
 scans = [22]  # list or array of scan numbers
-root_folder = "D:/data/P10_August2019_CDI/data/"
-save_dir = "D:/data/P10_August2019_CDI/test/"
+root_folder = "C:/Users/Jerome/Documents/data/dataset_P10_saxs/"
+data_dir = None
+# leave None to use the beamline default. It will look for the data at this location
+save_dir = root_folder + "test/"
 # images will be saved here, leave it to None otherwise
 # (default to data directory's parent)
-sample_name = [
-    "gold_2_2_2"
-]  # "S"  # # list of sample names. If only one name is indicated,
+sample_name = "gold_2_2_2"
+# "S"  # # list of sample names. If only one name is indicated,
 # it will be repeated to match the number of scans
 user_comment = ""  # string, should start with "_"
 debug = False  # set to True to see plots
-binning = [1, 1, 1]  # binning that will be used for phasing
+binning = [1, 2, 2]  # binning that will be used for phasing
 # (stacking dimension, detector vertical axis, detector horizontal axis)
+bin_during_loading = False  # True to bin during loading, require less memory
+frames_pattern = None
+# 1D array of int, of length data.shape[0]. If frames_pattern is 0 at index,
+# the frame at data[index] will be skipped, if 1 the frame will added to the stack.
+# Use this if you need to remove some frames and you know it in advance.
 ##############################
 # parameters used in masking #
 ##############################
@@ -68,7 +70,7 @@ background_plot = (
 ##############################################
 # parameters used in intensity normalization #
 ##############################################
-normalize_method = "skip"  # 'skip' for no normalization,
+normalize_method = "monitor"  # 'skip' for no normalization,
 # 'monitor' to use the default monitor, 'sum_roi' to normalize
 # by the intensity summed in normalize_roi
 normalize_roi = None
@@ -91,14 +93,10 @@ medfilt_order = 8  # for custom median filter,
 # parameters used when reloading processed data #
 #################################################
 reload_previous = False  # True to resume a previous masking (load data and mask)
-reload_orthogonal = (
-    False  # True if the reloaded data is already intepolated in an orthonormal frame
-)
-preprocessing_binning = [
-    1,
-    1,
-    1,
-]  # binning factors in each dimension of the binned data to be reloaded
+reload_orthogonal = False
+# True if the reloaded data is already intepolated in an orthonormal frame
+preprocessing_binning = (1, 1, 1)
+# binning factors in each dimension of the binned data to be reloaded
 ##################
 # saving options #
 ##################
@@ -106,17 +104,15 @@ save_rawdata = False  # save also the raw data when use_rawdata is False
 save_to_npz = True  # True to save the processed data in npz format
 save_to_mat = False  # True to save the processed data in mat format
 save_to_vti = False  # save the orthogonalized diffraction pattern to VTK file
-save_asint = (
-    False  # if True, the result will be saved as an array of integers (save space)
-)
+save_asint = False
+# if True, the result will be saved as an array of integers (save space)
+
 ###############################
 # beamline related parameters #
 ###############################
-beamline = (
-    "P10"  # name of the beamline, used for data loading and normalization by monitor
-)
-# supported beamlines: 'ID01', 'SIXS_2018', 'SIXS_2019', 'CRISTAL', 'P10'
-rocking_angle = "inplane"  # "outofplane" or "inplane"
+beamline = "P10_SAXS"
+# name of the beamline, used for data loading and normalization by monitor
+# supported beamlines: 'ID01', 'SIXS_2018', 'SIXS_2019', 'CRISTAL', 'P10', 'P10_SAXS'
 is_series = True  # specific to series measurement at P10
 
 custom_scan = False  # set it to True for a stack of images acquired without scan,
@@ -137,10 +133,8 @@ specfile_name = ""
 # detector related parameters #
 ###############################
 detector = "Eiger4M"  # "Eiger2M" or "Maxipix" or "Eiger4M"
-direct_beam = (
-    1349,
-    1321,
-)  # tuple of int (vertical, horizontal): position of the direct beam in pixels, in the
+direct_beam = (1349, 1321)
+# tuple of int (vertical, horizontal): position of the direct beam in pixels, in the
 # unbinned detector.
 # This parameter is important for gridding the data onto the laboratory frame.
 roi_detector = [
@@ -157,12 +151,9 @@ photon_filter = "loading"  # 'loading' or 'postprocessing',
 # if 'loading', it is applied before binning;
 # if 'postprocessing', it is applied at the end of the script before saving
 background_file = None  # root_folder + 'background.npz'  # non empty file path or None
-hotpixels_file = (
-    None  # root_folder + 'hotpixels_HS4670.npz'  # non empty file path or None
-)
-flatfield_file = (
-    None  # root_folder + "flatfield_maxipix_8kev.npz"  # non empty file path or None
-)
+hotpixels_file = root_folder + "hotpixels.npz"  # non empty file path or None
+flatfield_file = None
+# root_folder + "flatfield_maxipix_8kev.npz"  # non empty file path or None
 template_imagefile = "_master.h5"  # ''_data_%06d.h5'
 # template for ID01: 'data_mpx4_%05d.edf.gz' or 'align_eiger2M_%05d.edf.gz'
 # template for SIXS_2018: 'align.spec_ascan_mu_%05d.nxs'
@@ -176,9 +167,13 @@ template_imagefile = "_master.h5"  # ''_data_%06d.h5'
 ######################################################################
 use_rawdata = False  # False for using data gridded in laboratory frame
 # True for using data in detector frame
-correct_curvature = (
-    False  # True to correcture q values for the curvature of Ewald sphere
-)
+fill_value_mask = 1  # 0 (not masked) or 1 (masked).
+# It will define how the pixels outside of the data range are
+# processed during the interpolation. Because of the large number of masked pixels,
+# phase retrieval converges better if the pixels are not masked (0 intensity
+# imposed). The data is by default set to 0 outside of the defined range.
+correct_curvature = False
+# True to correcture q values for the curvature of Ewald sphere
 fit_datarange = True  # if True, crop the final array within data range,
 # avoiding areas at the corners of the window viewed from the top, data is circular,
 # but the interpolation window is rectangular, with nan values outside of data
@@ -434,7 +429,7 @@ setup = Setup(
     beamline=beamline,
     detector=detector,
     energy=energy,
-    rocking_angle=rocking_angle,
+    rocking_angle="inplane",
     distance=sdd,
     direct_beam=direct_beam,
     custom_scan=custom_scan,
@@ -483,10 +478,9 @@ for scan_idx, scan_nb in enumerate(scans, start=1):
         root_folder=root_folder,
         save_dir=save_dir,
         save_dirname=save_dirname,
-        verbose=True,
-        create_savedir=create_savedir,
         specfile_name=specfile_name,
         template_imagefile=template_imagefile,
+        data_dir=data_dir,
     )
 
     logfile = setup.create_logfile(
@@ -569,7 +563,7 @@ for scan_idx, scan_nb in enumerate(scans, start=1):
                     # along x outboard
                     del numz, numy, numx
         else:  # the data is in the detector frame
-            data, mask, frames_logical, monitor = pru.reload_cdi_data(
+            data, mask, frames_logical, monitor = cdi.reload_cdi_data(
                 logfile=logfile,
                 scan_number=scan_nb,
                 data=data,
@@ -587,11 +581,13 @@ for scan_idx, scan_nb in enumerate(scans, start=1):
         hotpix_array = util.load_hotpixels(hotpixels_file)
         background = util.load_background(background_file)
 
-        data, mask, frames_logical, monitor = pru.load_cdi_data(
+        data, mask, frames_logical, monitor = cdi.load_cdi_data(
             logfile=logfile,
             scan_number=scan_nb,
             detector=detector,
             setup=setup,
+            frames_pattern=frames_pattern,
+            bin_during_loading=bin_during_loading,
             flatfield=flatfield,
             hotpixels=hotpix_array,
             background=background,
@@ -740,7 +736,7 @@ for scan_idx, scan_nb in enumerate(scans, start=1):
             gc.collect()
 
             print("\nGridding the data in the orthonormal laboratory frame")
-            data, mask, q_values, frames_logical = pru.grid_cdi(
+            data, mask, q_values, frames_logical = cdi.grid_cdi(
                 data=data,
                 mask=mask,
                 logfile=logfile,
@@ -748,6 +744,7 @@ for scan_idx, scan_nb in enumerate(scans, start=1):
                 setup=setup,
                 frames_logical=frames_logical,
                 correct_curvature=correct_curvature,
+                fill_value=(0, fill_value_mask),
                 debugging=debug,
             )
 
@@ -992,9 +989,9 @@ for scan_idx, scan_nb in enumerate(scans, start=1):
         plt.disconnect(cid)
         plt.close(fig)
 
-        #############################################
-        # define mask
-        #############################################
+        ####################
+        # GUI for the mask #
+        ####################
         width = 0
         max_colorbar = 5
         flag_aliens = False
@@ -1064,7 +1061,7 @@ for scan_idx, scan_nb in enumerate(scans, start=1):
         print("\nFiltering isolated pixels")
         nb_pix = 0
         for idx in range(nz):  # filter only frames whith data (not padded)
-            data[idx, :, :], numb_pix, mask[idx, :, :] = pru.mean_filter(
+            data[idx, :, :], numb_pix, mask[idx, :, :] = util.mean_filter(
                 data=data[idx, :, :],
                 nb_neighbours=medfilt_order,
                 mask=mask[idx, :, :],
