@@ -16,7 +16,7 @@ import matplotlib.pyplot as plt
 from scipy.interpolate import interp1d
 from bcdi.experiment.detector import create_detector
 from bcdi.experiment.setup import Setup
-import bcdi.preprocessing.preprocessing_utils as pru
+import bcdi.preprocessing.bcdi_utils as bu
 import bcdi.graph.graph_utils as gu
 import bcdi.utils.utilities as util
 
@@ -31,10 +31,11 @@ Supported beamlines: ESRF ID01, PETRAIII P10, SOLEIL SIXS, SOLEIL CRISTAL.
 
 scan = 128
 root_folder = "D:/data/P10_2nd_test_isosurface_Dec2020/data_nanolab/"
+data_dir = None
+# leave None to use the beamline default. It will look for the data at this location
 sample_name = "PtNP1"  # string in front of the scan number in the folder name
-save_dir = (
-    root_folder + "dataset_1_newpsf/test/"
-)  # images will be saved here, leave it to None otherwise
+save_dir = root_folder + "dataset_1_newpsf/test/"
+# images will be saved here, leave it to None otherwise
 # (default to data directory's parent)
 save_mask = False  # set to True to save the mask
 debug = True  # True to see more plots
@@ -43,9 +44,8 @@ binning = (1, 1, 1)  # binning to apply to the data
 ###############################
 # beamline related parameters #
 ###############################
-beamline = (
-    "P10"  # name of the beamline, used for data loading and normalization by monitor
-)
+beamline = "P10"
+# name of the beamline, used for data loading and normalization by monitor
 # supported beamlines: 'ID01', 'SIXS_2018', 'SIXS_2019', 'CRISTAL', 'P10', 'NANOMAX'
 actuators = None  # {'rocking_angle': 'actuator_1_3'}
 # Optional dictionary that can be used to define the entries corresponding to
@@ -55,9 +55,8 @@ actuators = None  # {'rocking_angle': 'actuator_1_3'}
 custom_scan = False  # True for a stack of images acquired without scan,
 # e.g. with ct in a macro (no info in spec file)
 custom_images = np.arange(11353, 11453, 1)  # list of image numbers for the custom_scan
-custom_monitor = np.ones(
-    len(custom_images)
-)  # monitor values for normalization for the custom_scan
+custom_monitor = np.ones(len(custom_images))
+# monitor values for normalization for the custom_scan
 custom_motors = {
     "eta": np.linspace(16.989, 18.989, num=100, endpoint=False),
     "phi": 0,
@@ -94,9 +93,8 @@ roi_detector = [y_bragg - 200, y_bragg + 200, x_bragg - 200, x_bragg + 200]
 # leave None to use the full detector. Use with center_fft='skip'
 # if you want this exact size.
 peak_method = "max"  # Bragg peak determination: 'max', 'com' or 'maxcom'.
-normalize = (
-    "monitor"  # 'monitor' to return the default monitor values, 'skip' to do nothing
-)
+normalize = "monitor"
+# 'monitor' to return the default monitor values, 'skip' to do nothing
 high_threshold = 500000  # everything above will be considered as hotpixel
 hotpixels_file = ""  # root_folder + 'hotpixels_cristal.npz'
 # root_folder + 'merlin_mask_190222_14keV.h5'  #
@@ -182,10 +180,9 @@ setup.init_paths(
     root_folder=root_folder,
     save_dir=save_dir,
     save_dirname=save_dirname,
-    verbose=True,
-    create_savedir=True,
     specfile_name=specfile_name,
     template_imagefile=template_imagefile,
+    data_dir=data_dir,
 )
 
 logfile = setup.create_logfile(
@@ -196,7 +193,7 @@ logfile = setup.create_logfile(
 #################
 # load the data #
 #################
-data, mask, monitor, frames_logical = pru.load_data(
+data, mask, monitor, frames_logical = setup.diffractometer.load_check_dataset(
     logfile=logfile,
     scan_number=scan,
     detector=detector,
@@ -229,7 +226,7 @@ if data.ndim == 3:
     rocking_curve = np.zeros(numz)
 
     z0, y0, x0 = tuple(
-        map(lambda x: int(np.rint(x)), pru.find_bragg(data, peak_method=peak_method))
+        map(lambda x: int(np.rint(x)), bu.find_bragg(data, peak_method=peak_method))
     )
 
     if x_bragg is None:  # Bragg peak position not defined by the user, use the max
@@ -292,7 +289,7 @@ if data.ndim == 3:
     data = data.sum(axis=0)  # concatenate along the axis of the rocking curve
     title = f"data.sum(axis=0)   peak method={peak_method}\n"
 else:  # 2D
-    _, y0, x0 = pru.find_bragg(data, peak_method=peak_method)
+    _, y0, x0 = bu.find_bragg(data, peak_method=peak_method)
     peak_int = int(data[y0, x0])
     print(
         f"Bragg peak (indices in the eventually binned ROI) at (y, x): {y0}, {x0},"
