@@ -21,7 +21,7 @@ import sys
 import bcdi.graph.graph_utils as gu
 import bcdi.utils.utilities as util
 import bcdi.xcca.xcca_utils as xcca
-import bcdi.facet_recognition.facet_utils as fu
+import bcdi.postprocessing.facet_recognition as fu
 
 helptext = """
 Calculate the angular cross-correlation in a 3D reciprocal space dataset over a range
@@ -114,11 +114,11 @@ def main(calc_self, user_comment):
         raise ValueError("at least 2 values are needed for q_range")
 
     print("the CCF map will be calculated for {:d} q values: ".format(len(q_range)))
-    for idx in range(len(q_range)):
+    for idx, item in enumerate(q_range):
         if calc_self:
-            print("q1 = {:.3f}  q2 = {:.3f}".format(q_range[idx], q_range[idx]))
+            print(f"q1 = {item:.3f}  q2 = {item:.3f}")
         else:
-            print("q1 = {:.3f}  q2 = {:.3f}".format(q_range[0], q_range[idx]))
+            print("q1 = {:.3f}  q2 = {:.3f}".format(q_range[0], item))
     warnings.filterwarnings("ignore")
 
     ###################
@@ -204,10 +204,8 @@ def main(calc_self, user_comment):
     dq = min(qx[1] - qx[0], qz[1] - qz[0], qy[1] - qy[0])
 
     theta_phi_int = {}  # create dictionnary
-    dict_fields = []
-    [
-        dict_fields.append("q" + str(idx + 1)) for idx in range(len(q_range))
-    ]  # ['q1', 'q2', 'q3', ...]
+    dict_fields = ["q" + str(idx + 1) for idx, _ in enumerate(q_range)]
+    # ['q1', 'q2', 'q3', ...]
     nb_points = []
 
     for counter, q_value in enumerate(q_range):
