@@ -21,7 +21,7 @@ import sys
 import bcdi.graph.graph_utils as gu
 import bcdi.utils.utilities as util
 import bcdi.xcca.xcca_utils as xcca
-import bcdi.facet_recognition.facet_utils as fu
+import bcdi.postprocessing.facet_recognition as fu
 
 helptext = """
 Calculate the angular cross-correlation in a 3D reciprocal space dataset at the same
